/**************************************************************************\
*          Copyright (c) 1992 INRIA Sophia Antipolis, FRANCE.              *
*                                                                          *
* Permission to use, copy, modify, and distribute this material for any    *
* purpose and without fee is hereby granted, provided that the above       *
* copyright notice and this permission notice appear in all copies.        *
* WE MAKE NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS     *
* MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS   *
* OR IMPLIED WARRANTIES.                                                   *
\**************************************************************************/
/**************************************************************************\
* 	                						   *
*  File              : athena.c    	                		   *
*  Author            : Thierry Turletti					   *
*  Last Modification : 93/19/1                                             *
*--------------------------------------------------------------------------*
*  Description       : Athena toolkit.                                     *
*                                                                          *
*--------------------------------------------------------------------------*
*        Name	        |    Date   |          Modification                *
*--------------------------------------------------------------------------*
*                                                                          *
\**************************************************************************/



#include <stdio.h>
#include <string.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>	

#include <X11/Xaw/Command.h>	
#include <X11/Xaw/Label.h>	
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Cardinals.h>	
#include <X11/Xaw/Scrollbar.h>
#include <X11/Xaw/Sme.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/Dialog.h>

#define BOOLEAN     unsigned char
#ifndef TRUE
#define TRUE                    1
#endif /* TRUE */
#ifndef FALSE                   
#define FALSE                   0
#endif /* FALSE */



void DestroyWarning(w, client_data, call_data)
     Widget w;
     XtPointer client_data, call_data;
{
  Widget popup;

  popup = XtParent((Widget) client_data);
  XtDestroyWidget(popup);
}
  


void ShowWarning(msg)
     char *msg;
{
  Arg         args[5];
  Widget      popup, dialog;
  Position    x, y;
  Dimension   width, height;
  Cardinal    n;
  extern Widget main_form, msg_error;

  n = 0;
  XtSetArg(args[0], XtNwidth, &width); n++;
  XtSetArg(args[1], XtNheight, &height); n++;
  XtGetValues(main_form, args, n);
  XtTranslateCoords(main_form, (Position) (width/2), (Position) (height/2),
		    &x, &y);

  n = 0;
  XtSetArg(args[n], XtNx, x); n++;
  XtSetArg(args[n], XtNy, y); n++;

  popup = XtCreatePopupShell("Warning", transientShellWidgetClass, main_form,
			     args, n);

  n = 0;
  XtSetArg(args[n], XtNlabel, msg); n++;
  msg_error = XtCreateManagedWidget("MsgError", dialogWidgetClass, popup,
				    args, n);

  XawDialogAddButton(msg_error, "OK", DestroyWarning, 
		     (XtPointer) msg_error);
  XtPopup(popup, XtGrabNone);
}


void MarkMenu(widget)
     Widget widget;
{
  Arg args[1];
  extern Pixmap mark;

  XtSetArg(args[0], XtNleftBitmap, mark);
  XtSetValues(widget, args, 1);
}


void UnMarkMenu(widget)
     Widget widget;
{
  Arg args[1];

  XtSetArg(args[0], XtNleftBitmap, None);
  XtSetValues(widget, args, 1);
}


void ChangeLabel(widget, name)
     Widget widget;
     char *name;
{
  Arg args[1];

  XtSetArg (args[0], XtNlabel, name);
  XtSetValues (widget, args, 1);
}


void ChangeAndMappLabel(widget, name)
     Widget widget;
     char *name;
{
  Arg args[2];
  
  XtSetArg (args[0], XtNlabel, name);
  XtSetArg (args[1], XtNmappedWhenManaged, TRUE);
  XtSetValues (widget, args, 2);
}


void ReverseColor(widget)
     Widget widget;
{
  Arg args[10];
  Pixel bg_color, fg_color;
  int n;

  n = 0;
  XtSetArg (args[n], XtNbackground, &bg_color); n++;
  XtSetArg (args[n], XtNforeground, &fg_color); n++;
  XtGetValues (widget, args, n);
  
  n = 0;
  XtSetArg (args[n], XtNbackground, fg_color); n++;
  XtSetArg (args[n], XtNforeground, bg_color); n++;
  XtSetValues (widget, args, n);
}



void SetSensitiveVideoTuning(value)
     BOOLEAN value;
{
  extern Widget form_brightness, form_contrast, button_default;

  XtSetSensitive(form_brightness, value);
  XtSetSensitive(form_contrast, value);
  XtSetSensitive(button_default, value);
}



void SetSensitiveVideoMenu(value)
     BOOLEAN value;
{
  register int i;
  extern Widget menu_video[3];

#ifdef VIDEOPIX
  for(i=0; i<3; i++)
#else
  for(i=0; i<2; i++)
#endif
    XtSetSensitive(menu_video[i], value);
}



void CreateScale(form, scroll, name, value, parent, FormScale, 
			NameScale, val_default)
     Widget *form, *scroll, *name, *value, parent;
     char *FormScale, *NameScale;
     int val_default;
{
  int n;
  Arg args[10];
  float top;
  XtArgVal *l_top;
  char value_default[4];

  sprintf(value_default, "%d", val_default);
  /*
  * Create the form for the scale.
  */
  *form = XtCreateManagedWidget(FormScale, formWidgetClass, parent, NULL, 0);

  n = 0;
  top = 0.5;
  if(sizeof(float) > sizeof(XtArgVal)){
    XtSetArg(args[n], XtNtopOfThumb, &top); n++;
  }else{
    l_top = (XtArgVal *) &top;
    XtSetArg(args[n], XtNtopOfThumb, *l_top); n++;
  }
  *scroll = XtCreateManagedWidget("ScrollScale", scrollbarWidgetClass,
				  *form, args, n);
  top = (float)val_default / 100.0;
  XawScrollbarSetThumb(*scroll, top, 0.05);

  n = 0;
  XtSetArg(args[n], XtNfromVert, *scroll); n++;
  *name = XtCreateManagedWidget(NameScale, labelWidgetClass, *form, 
				     args, n);
  n = 0;
  XtSetArg(args[n], XtNlabel, value_default); n++;
  XtSetArg(args[n], XtNfromHoriz, *name); n++;
  XtSetArg(args[n], XtNfromVert, *scroll); n++;
  *value = XtCreateManagedWidget("ValueScale", labelWidgetClass, *form, 
				 args, n);

}
  


void InitScale(scroll, value, val_default)
     Widget scroll, value;
     int val_default;
{
  Arg args[1];
  float top;
  char str_default[4];

  sprintf(str_default, "%d", val_default);
  XtSetArg(args[0], XtNlabel, str_default);
  XtSetValues(value, args, 1);

  top = (float)val_default / 100.0;
  XawScrollbarSetThumb(scroll, top, 0.05);
}




