/**************************************************************************\
*          Copyright (c) 1992 INRIA Sophia Antipolis, FRANCE.              *
*                                                                          *
* Permission to use, copy, modify, and distribute this material for any    *
* purpose and without fee is hereby granted, provided that the above       *
* copyright notice and this permission notice appear in all copies.        *
* WE MAKE NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS     *
* MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS   *
* OR IMPLIED WARRANTIES.                                                   *
\**************************************************************************/
/**************************************************************************\
*                                                                          *
*  File    : huffman_coder.h                                               *
*  Date    : May 1992                                                      *
*  Author  : Thierry Turletti                                              *
*--------------------------------------------------------------------------*
*  Description :  Huffman mot_code arrays used by H.261 coder only.        *
*                                                                          *
*                   - state_mba     -> Macroblock address.                 *
*                   - state_typem   -> Type information.                   *
*                   - state_cbc     -> Coded block pattern.                *
*                   - state_dvm     -> Motion vector.                      *
*                   - state_tcoeff  -> DCT coefficient.                    *
*                                                                          *
\**************************************************************************/


static mot_code tab_mba[] = {
      {0,"00000001111"}, {1,"1"}, {2,"011"}, 
      {3,"010"}, {4,"0011"}, {5,"0010"}, 
      {6,"00011"}, {7,"00010"}, {8,"0000111"}, 
      {9,"0000110"}, {10,"00001011"}, {11,"00001010"}, 
      {12,"00001001"}, {13,"00001000"}, {14,"00000111"}, 
      {15,"00000110"}, {16,"0000010111"}, {17,"0000010110"}, 
      {18,"0000010101"}, {19,"0000010100"}, {20,"0000010011"}, 
      {21,"0000010010"}, {22,"00000100011"}, {23,"00000100010"}, 
      {24,"00000100001"}, {25,"00000100000"}, {26,"00000011111"}, 
      {27,"00000011110"}, {28,"00000011101"}, {29,"00000011100"}, 
      {30,"00000011011"}, {31,"00000011010"}, {32,"00000011001"}, 
      {33,"00000011000"}
    };

static mot_code tab_cbc[] = {
      {1,"01011"}, {2,"01001"}, {3,"001101"}, 
      {4,"1101"}, {5,"0010111"}, {6,"0010011"}, 
      {7,"00011111"}, {8,"1100"}, {9,"0010110"}, 
      {10,"0010010"}, {11,"00011110"}, {12,"10011"}, 
      {13,"00011011"}, {14,"00010111"}, {15,"00010011"}, 
      {16,"1011"}, {17,"0010101"}, {18,"0010001"}, 
      {19,"00011101"}, {20,"10001"}, {21,"00011001"}, 
      {22,"00010101"}, {23,"00010001"}, {24,"001111"}, 
      {25,"00001111"}, {26,"00001101"}, {27,"000000011"}, 
      {28,"01111"}, {29,"00001011"}, {30,"00000111"}, 
      {31,"000000111"}, {32,"1010"}, {33,"0010100"}, 
      {34,"0010000"}, {35,"00011100"}, {36,"001110"}, 
      {37,"00001110"}, {38,"00001100"}, {39,"000000010"}, 
      {40,"10000"}, {41,"00011000"}, {42,"00010100"}, 
      {43,"00010000"}, {44,"01110"}, {45,"00001010"}, 
      {46,"00000110"}, {47,"000000110"}, {48,"10010"}, 
      {49,"00011010"}, {50,"00010110"}, {51,"00010010"}, 
      {52,"01101"}, {53,"00001001"}, {54,"00000101"}, 
      {55,"000000101"}, {56,"01100"}, {57,"00001000"}, 
      {58,"00000100"}, {59,"000000100"}, {60,"111"}, 
      {61,"01010"}, {62,"01000"}, {63,"001100"}
    };

static mot_code tab_dvm[] = {
      {-16,"00000011001"}, {-15,"00000011011"}, {-14,"00000011101"}, 
      {-13,"00000011111"}, {-12,"00000100001"}, {-11,"00000100011"}, 
      {-10,"0000010011"}, {-9,"0000010101"}, {-8,"0000010111"}, 
      {-7,"00000111"}, {-6,"00001001"}, {-5,"00001011"}, 
      {-4,"0000111"}, {-3,"00011"}, {-2,"0011"}, 
      {-1,"011"}, {0,"1"}, {1,"010"}, 
      {2,"0010"}, {3,"00010"}, {4,"0000110"}, 
      {5,"00001010"}, {6,"00001000"}, {7,"00000110"}, 
      {8,"0000010110"}, {9,"0000010100"}, {10,"0000010010"}, 
      {11,"00000100010"}, {12,"00000100000"}, {13,"00000011110"}, 
      {14,"00000011100"}, {15,"00000011010"}
    };

static mot_code tab_tcoeff[] = {
      {-1,"10"}, {0,"000001"}, {1,"11"}, 
      {2,"0100"}, {3,"00101"}, {4,"0000110"}, 
      {5,"00100110"}, {6,"00100001"}, {7,"0000001010"}, 
      {8,"000000011101"}, {9,"000000011000"}, {10,"000000010011"}, 
      {11,"000000010000"}, {12,"0000000011010"}, {13,"0000000011001"}, 
      {14,"0000000011000"}, {15,"0000000010111"}, {17,"011"}, 
      {18,"000110"}, {19,"00100101"}, {20,"0000001100"}, 
      {21,"000000011011"}, {22,"0000000010110"}, {23,"0000000010101"}, 
      {33,"0101"}, {34,"0000100"}, {35,"0000001011"}, 
      {36,"000000010100"}, {37,"0000000010100"}, {49,"00111"}, 
      {50,"00100100"}, {51,"000000011100"}, {52,"0000000010011"}, 
      {65,"00110"}, {66,"0000001111"}, {67,"000000010010"}, 
      {81,"000111"}, {82,"0000001001"}, {83,"0000000010010"}, 
      {97,"000101"}, {98,"000000011110"}, {113,"000100"}, 
      {114,"000000010101"}, {129,"0000111"}, {130,"000000010001"}, 
      {145,"0000101"}, {146,"0000000010001"}, {161,"00100111"}, 
      {162,"0000000010000"}, {177,"00100011"}, {193,"00100010"}, 
      {209,"00100000"}, {225,"0000001110"}, {241,"0000001101"}, 
      {257,"0000001000"}, {273,"000000011111"}, {289,"000000011010"}, 
      {305,"000000011001"}, {321,"000000010111"}, {337,"000000010110"}, 
      {353,"0000000011111"}, {369,"0000000011110"}, {385,"0000000011101"}, 
      {401,"0000000011100"}, {417,"0000000011011"}
    };
