/**************************************************************************
*          Copyright (c) 1992 INRIA Sophia Antipolis, FRANCE.              *
*                                                                          *
* Permission to use, copy, modify, and distribute this material for any    *
* purpose and without fee is hereby granted, provided that the above       *
* copyright notice and this permission notice appear in all copies.        *
* WE MAKE NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS     *
* MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS   *
* OR IMPLIED WARRANTIES.                                                   *
\**************************************************************************/
/**************************************************************************\
*                                                                          *
*  File              : video_coder.c                                       *
*  Author            : Thierry Turletti                                    *
*  Last modification : 12/17/92                                            *
*--------------------------------------------------------------------------*
*  Description :  Video Coder.                                             *
*                                                                          *
*--------------------------------------------------------------------------*
* Modified by Winston Dang  wkd@Hawaii.Edu	12/8/92                    *
* alterations were made for speed purposes.                                *
\**************************************************************************/

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <math.h>
#include <memory.h>
#include "h261.h"
#include "huffman.h"
#include "huffman_coder.h"
#include "quantizer.h"
#include "protocol.h"

#ifdef VIDEOPIX
#include <vfc_lib.h>
#endif

#define MAXBSIZE               8192
#define LIG_MAX                 288
#define COL_MAX                 352
#define LIG_MAX_DIV2            144
#define COL_MAX_DIV2            176
#define NBLOCKMAX              1584


/* mod- change + to | operation - much faster  also eliminated the 7-i op */
#define put_bit(octet,bit,i,j) \
    (!j ? (octet[(i)++] |=bit, octet[(i)]=0, j=7) : \
     (octet[i] |=(bit<< j--)))

/* mod -pointers are faster than array lookup */
#define put_bit_ptr(octet,bit,i,j) \
    (!j ? (*octet++ |=bit, *octet=0, i++, j=7) : \
     (*octet |=(bit<< j--)))


static unsigned char ZIG[] =
  { /* Raw coefficient block in ZIGZAG scan order */
   0,0,1,2,1,0,0,1,2,3,4,3,2,1,0,0,1,2,3,4,5,6,5,4,3,2,1,0,0,1,2,3,
   4,5,6,7,7,6,5,4,3,2,1,2,3,4,5,6,7,7,6,5,4,3,4,5,6,7,7,6,5,6,7,7
  };

static unsigned char ZAG[] =
  { /* Column coefficient block in ZIGZAG scan order */
   0,1,0,0,1,2,3,2,1,0,0,1,2,3,4,5,4,3,2,1,0,0,1,2,3,4,5,6,7,6,5,4,
   3,2,1,0,1,2,3,4,5,6,7,7,6,5,4,3,2,3,4,5,6,7,7,6,5,4,5,6,7,7,6,7
  };


FILE *F_loss;
extern BOOLEAN STAT_MODE;



int declare_listen(num_port)
    int num_port;

  {
    int size_of_ad_in;
    struct sockaddr_in ad_in;
    int sock;

    bzero((char *)&ad_in, sizeof(ad_in));
    ad_in.sin_family = AF_INET;
    ad_in.sin_port = htons((u_short)num_port);
    ad_in.sin_addr.s_addr = INADDR_ANY;

    if((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){
      fprintf(stderr,"declare_listen : cannot create socket-controle UDP\n");
      exit(1);
    }
    size_of_ad_in = sizeof(ad_in);
    if (bind(sock,(struct sockaddr *) &ad_in, size_of_ad_in) < 0){
      perror("declare_listen'bind");
      close(sock);
      exit(1);
    }
    if(!num_port){
      getsockname(sock,(struct sockaddr *)&ad_in,&size_of_ad_in);
      fprintf(stderr, "Listening at %d port number\n", ad_in.sin_port);
    }
    return(sock);
  }    




int ReceivePacket(sock, buf, from_gob, to_gob, cif_number_lost)
     int sock;
     u_char *buf;
     u_char *from_gob, *to_gob;
     int *cif_number_lost;

  {
    struct sockaddr_in adr;
    struct hostent *host;
    int er;
    int result = -1;

    er = sizeof(adr);
    if (recvfrom(sock, (char *)buf, 10, 0, (struct sockaddr *)&adr, &er) < 0){
      perror("receive from");
      exit(0);
    }

    /* First Header byte is : F=1 if P_FULL_INTRA_REQUEST.
    *  -----------------      L=1 if P_SEND_INFO_PACKET_LOST.
    *  |F|L|C|C|X|X|X|X|     CC= number of CIF lost if Super CIF. 
    *  -----------------   AAAA and BBBB = GOB numbers.
    *  |A|A|A|A|B|B|B|B|   GOB are lost from AAAA to BBBB.
    *  -----------------  
    *  | image number  |  <-- lost image number if P_SEND_INFO_PACKET_LOST.
    *  -----------------
    */

    if(buf[0] & P_FULL_INTRA_REQUEST)
      result = 256;
    else{
      if(buf[0] & P_SEND_INFO_PACKET_LOST){
	result = (int)buf[2]; /* lost image number */
	*from_gob = (buf[1] >> 4) & 0x0f;
	*to_gob = buf[1] & 0x0f;
	*cif_number_lost = (buf[0] >> 4) & 0x03;
      }else
	fprintf(stderr, 
	 "VideoDecode'ReceivePacket: Received an incorrect header packet %d\n",
		(int)buf[1]);
    }
    if(STAT_MODE){
      host=gethostbyaddr((char *)(&(adr.sin_addr)), sizeof(adr.sin_addr),
			 adr.sin_family);
      if((result>=0) && (result<256)){
	if(*from_gob != *to_gob)
	  fprintf(F_loss, 
		  "** From %s : image #%d, CIF #%d, GOB %d to %d are lost\n", 
		  host->h_name, result, *cif_number_lost, *from_gob, *to_gob);
	else
	  fprintf(F_loss, 
		  "** From %s : image #%d, CIF #%d, GOB #%d is lost\n", 
		  host->h_name, result, *cif_number_lost, *from_gob);
      }else{
	if(result==256)
	  fprintf(F_loss, "From %s : Full INTRA encoding required\n", 
		  host->h_name);
      }
    }
#ifdef DEBUG
    if(result != 256){
      if(*from_gob != *to_gob)
	fprintf(stderr, "** image #%d, CIF #%d, GOB %d to %d are lost\n", 
	       result, *cif_number_lost, *from_gob, *to_gob);
      else
	fprintf(stderr, "** image #%d, CIF #%d, GOB #%d is lost\n", 
	       result, *cif_number_lost, *from_gob);
    }
#endif
    return(result);
  }



int NbPacketsLost(sock, PARTIAL, ForceGobEncoding, image_number, INC_NG)
     int sock;
     BOOLEAN *PARTIAL;
     u_char ForceGobEncoding[13];
     int image_number;
  {
    int mask, num, nb_lost=0;
    int temp;
    u_char buf[1], from_gob, to_gob;
    int cif_number_lost; /* unused here */
    static int mask0;
    static struct timeval tim_out;
    static BOOLEAN first=TRUE;

    if(first){
      first = FALSE;
      mask0 = 1 << sock;
      tim_out.tv_sec = 0;
      tim_out.tv_usec = 0;
    }
    bzero((char *) ForceGobEncoding, 13);
    mask = mask0;
    *PARTIAL=TRUE;
    while(select(sock+1, (fd_set *)&mask, (fd_set *)0, (fd_set *)0, &tim_out) 
       != 0){
      num=ReceivePacket(sock, buf, &from_gob, &to_gob, &cif_number_lost);
      if(*PARTIAL){
	if(num==256){
	  *PARTIAL=FALSE;
	}else{
	  register int i;

	  if(num>=0){
	    temp = image_number - num;
	    if(temp>0){
	      for(i=from_gob; i<=to_gob; i+=INC_NG)
		if(ForceGobEncoding[i]<temp){
		  nb_lost ++;
		  ForceGobEncoding[i]=temp;
		}
	    }else{
	      temp = 256 - num + image_number;
	      for(i=from_gob; i<=to_gob; i+=INC_NG)
		if(ForceGobEncoding[i]<temp){
		  nb_lost ++;
		  ForceGobEncoding[i]=temp;
		}
	    }
	  }
	}
      }
      mask = mask0;
    }
#ifdef DEBUG
    if(nb_lost>0){
      register int i;
      fprintf(stderr, "ForceGob= ");
      for(i=1; i<13; i++)
	fprintf(stderr, "%d, ",ForceGobEncoding[i]);
      fprintf(stderr, "\n");
    }
#endif
    return(nb_lost);
  }


  
int NbPacketsLost4(sock, PARTIAL, ForceGobEncoding, image_number, INC_NG)
     int sock;
     BOOLEAN *PARTIAL;
     BOOLEAN ForceGobEncoding[][13];
     int image_number;
  {
    int mask, num, nb_lost=0;
    int temp;
    u_char buf[1], from_gob, to_gob;
    int cif_number_lost;
    static int mask0;
    static struct timeval tim_out;
    static BOOLEAN first=TRUE;

    if(first){
      first = FALSE;
      mask0 = 1 << sock;
      tim_out.tv_sec = 0;
      tim_out.tv_usec = 0;
    }
    bzero((char *) ForceGobEncoding, 52);
    mask = mask0;
    *PARTIAL=TRUE;
    while(select(sock+1, (fd_set *)&mask, (fd_set *)0, (fd_set *)0, &tim_out) 
       != 0){
      
      num=ReceivePacket(sock, buf, &from_gob, &to_gob, &cif_number_lost);
      if(*PARTIAL){
	if(num==256){
	  *PARTIAL=FALSE;
	}else{
	  register int i;

	  if(num >= 0){
	    temp = image_number - num;
	    if(temp > 0){
	      for(i=from_gob; i<=to_gob; i++)
		if(ForceGobEncoding[cif_number_lost][i] < temp){
		  nb_lost ++;
		  ForceGobEncoding[cif_number_lost][i]=temp;
		}
	    }else{
	      temp = 256 - num + image_number;
	      for(i=from_gob; i<=to_gob; i++)
		if(ForceGobEncoding[cif_number_lost][i] < temp){
		  nb_lost ++;
		  ForceGobEncoding[cif_number_lost][i]=temp;
		}
	    }
	  }
	}
      }
      mask = mask0;
    }

    return(nb_lost);
  }



void coffee_break(delay)
     long delay;
{
  struct timeval timer;
  static BOOLEAN First=TRUE;
  static int sock;

  if(First){
    if((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){
      perror("coffee_break creation socket ");
      return;
    }
    First = FALSE;
  }    
  timer.tv_sec = (delay/1000000);
  timer.tv_usec = (delay % 1000000);
  select(sock+1, (fd_set *)0, (fd_set *)0, (fd_set *)0, &timer);
  return;

}



encode_h261(group, port, ttl, new_Y, new_Cb, new_Cr, image_coeff, NG_MAX, 
	    sock_recv, FIRST, COLOR, format, rate_control, rate_max, fd)
     char *group, *port;
     u_char ttl;       /* ttl value */
     u_char new_Y[][COL_MAX], new_Cb[][COL_MAX_DIV2], new_Cr[][COL_MAX_DIV2];
     IMAGE_CIF image_coeff;
     int NG_MAX;       /* Maximum NG number */
     int sock_recv;    /* socket used for loss messages */
     BOOLEAN FIRST;    /* True if it is the first image */
     BOOLEAN COLOR;    /* True if color encoded */
     int format;       /* VFC_NTSC or VFC_PAL */
     int rate_control; /* WITHOUT_CONTROL or REFRESH_CONTROL 
			  or QUALITY_CONTROL */
     int rate_max;     /* Maximum bandwidth allowed */
     int fd;           /* Pipe son to father for bandwidth & rate display */

  {
    int lig, col, ligdiv2, coldiv2;
    int NG, MBA, INC_NG, CIF;
    GOB *gob;
    MACRO_BLOCK *mb;
    static u_char old_Y[LIG_MAX][COL_MAX];
    static u_char old_Cb[LIG_MAX_DIV2][COL_MAX_DIV2];
    static u_char old_Cr[LIG_MAX_DIV2][COL_MAX_DIV2];
    int diff_Y[LIG_MAX][COL_MAX];
    int diff_Cb[LIG_MAX_DIV2][COL_MAX_DIV2];
    int diff_Cr[LIG_MAX_DIV2][COL_MAX_DIV2];
    int cc, quant_mb;
    int k;
    int bnul, mbnul;
    int cmax0, cmax1, cmax2, cmax3, cmax4, cmax5;
    int *tab_quant1, *tab_quant2;
    u_char chaine_codee[T_MAX];
    int i, j=7;
    static int lwrite;
    struct timeval realtime;
    static double inittime, synctime;
    static int RT;
    static int n_padd[8]={4, 1, 6, 3, 0, 5, 2, 7};
    static int p_padd[8]={0, 5, 2, 7, 4, 1, 6, 3};
    static int LIG[16]={0, 2, 0, 1, 3, 1, 3, 2, 0, 3, 2, 1, 3, 0, 2, 1};
    static int COL[16]={0, 2, 3, 1, 0, 2, 3, 1, 2, 1, 3, 0, 2, 1, 0, 3};
    static int MASK_CBC[6]={0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
    static int n_im=0;
    int lig11, lig12, lig21, lig22, col11, col12, col21, col22;
    int lig11div2, lig12div2, col11div2, col12div2;
    static int Nblock=0;
    int max_coeff, max;
    static int cpt=0;
    static int packet_number=0;
    static int image_number=0;
    static BOOLEAN PARTIAL=FALSE;
    BOOLEAN GobEncoded[12];
    static u_char ForceGobEncoding[13];
    BOOLEAN FFCT8x8=TRUE;
    int nb_lost;
    extern int ffct_4x4(), ffct_8x8();
    extern int ffct_intra_4x4(), ffct_intra_8x8();
    int (*ffct)()=ffct_8x8;
    int (*ffct_intra)()=ffct_intra_8x8;
    int nb_packet=1;
    int first_packet=packet_number;
    int nblock_cif_max=1200;  /* CIF  : if nb(blocks) detected > this value */
    int nblock_qcif_max=300; /* QCIF :    then use INTRA mode              */
    BOOLEAN VIDEO_PACKET_HEADER; /* True if it is set */
    static int first_byte;
    static int BperMB, full_cbc;
    static struct sockaddr_in addr;
    static int sock, len_addr;
    static int max_value; /* A block is forced every max_value images */
    static BOOLEAN LIMIT_MBA; /* True if QCIF and NTSC formats */
    int max_mba;
    int force_value;
    static int offsetY, offsetC; /* Useful to pointers alignment */
    static int threshold=20; /* For motion estimation */
    static int QUANT_FIX=3;
    static int length_image=0;
    static int len_data=11;
    static double fps, bandwidth;
    char data[11];
    double rate;

    /* Header video packet format : 3 bytes
    *  -----------------  00: Video Packet header data.
    *  |0|0|I|P|S|N|N|C|  I : 1 if Full Intra Encoded image.
    *  -----------------  P : 1 if PAL FORMAT.
    *  | packet number |  S : 1 if CIF4, else 0.
    *  -----------------  NN: if CIF4, refers the CIF in the CIF4 (0 to 3). 
    *  | image  number |  C : 1 if color encoded, else 0.
    *  -----------------
    */

    if(FIRST){
#ifdef VIDEOPIX
      first_byte = (COLOR ? (format==VFC_PAL ? 0x11 : 0x01) : 
		    (format==VFC_PAL ? 0x10 : 0));
#else
      first_byte = (COLOR ? 0x11 : 0x10);
#endif
      if(COLOR){
	BperMB = 6;
	full_cbc = 63;
      }else{
	BperMB = 4;
	full_cbc = 60;
      }
    }
    if(!PARTIAL)
      chaine_codee[0] = first_byte | 0x20;
    else
      chaine_codee[0] = first_byte;
    chaine_codee[1] = packet_number;
    chaine_codee[2] = image_number;
    chaine_codee[3] = (u_char) 0;
    i = 3;
    VIDEO_PACKET_HEADER=TRUE;
    CIF=(NG_MAX != 5);
    INC_NG=(CIF ? 1 : 2);
    

    /*************\
    * IMAGE LAYER *
    \*************/
    gettimeofday(&realtime, (struct timezone *)NULL);
    if(FIRST){
      /* some initializations ... */
      offsetY = COL_MAX - 8;
      offsetC = COL_MAX_DIV2 - 8;
      max_value = (CIF ? 50 : 100);
#ifdef VIDEOPIX
      LIMIT_MBA = ((!CIF && (format==VFC_NTSC)) ? TRUE : FALSE);
#else
      LIMIT_MBA = FALSE;
#endif
      inittime = realtime.tv_sec + realtime.tv_usec/1000000.0;
      RT=0;
      bzero((char *)image_coeff, sizeof(IMAGE_CIF));
      sock = CreateSendSocket(&addr, &len_addr, port, group, &ttl);
      FIRST = FALSE;
    }else{
      synctime = realtime.tv_sec + realtime.tv_usec/1000000.0;
      RT=((RT+(int)(33*(synctime-inittime))) & 0x1f);
#ifdef DEBUG
      fprintf(stderr, "Encoding time: %d ms\n", 
	      (int)(1000*(synctime-inittime)));
#endif

      if(rate_control == QUALITY_CONTROL){
	rate = (double)(0.008*length_image) / (synctime-inittime);
	if(rate > (double)rate_max){
	  if(threshold <= 30)
	    threshold += 5;
	  if(QUANT_FIX <= 11)
	    QUANT_FIX +=2;
#ifdef DEBUG
	  fprintf(stderr, "rate ko: %2.3f, threshold: %d, quant: %d\n", 
		  rate, threshold, QUANT_FIX);
#endif
	}else{
	  if(threshold > 15)
	    threshold -= 5;
	  if(QUANT_FIX > 3)
	    QUANT_FIX -= 2;
#ifdef DEBUG
	  fprintf(stderr, "rate ok: %2.3f, threshold: %d, quant: %d\n", 
		  rate, threshold, QUANT_FIX);
#endif
	}
      }
      
      fps += (1.0 / (synctime-inittime));
      bandwidth += ((double)(0.008*length_image) / (synctime-inittime));
      if((cpt > 0) && (cpt % 3 == 0)){
	bzero(data, len_data);
	sprintf(data, "%2.1f", bandwidth/3.0);
	sprintf(&data[6], "%2.1f", fps/3.0);
	if((lwrite=write(fd, data, len_data)) != len_data){
#ifdef DEBUG
	  perror("encode_h261: Cannot send bandwidth and rate data");
#endif
	}
	fps = 0.0;
	bandwidth = 0.0;
      }
      Nblock = 0;
      length_image = 0;
      inittime=synctime;
    }
#ifdef DEBUG
    fprintf(stderr, "QUANT: %d, RT: %d\n", QUANT_FIX, RT);
#endif
    put_value(&chaine_codee[i],16,20,&i,&j); /* CDI */
    put_value(&chaine_codee[i],RT,5,&i,&j); /* RT */
    if(CIF)
	put_value(&chaine_codee[i],4,6,&i,&j); /* TYPEI = CIF */
    else
	put_value(&chaine_codee[i],0,6,&i,&j); /* TYPEI = QCIF */
    put_bit(chaine_codee,0,i,j); /* ISI1 */

    for(k=0; k<NG_MAX; k+=INC_NG)
      GobEncoded[k]=TRUE;

    
    if(PARTIAL){
      lig11=LIG[n_im];
      col11=COL[n_im];
      n_im=(n_im+1)%16;
      lig12=lig11+4;
      col12=col11+4;
      lig21=lig11+8;
      lig22=lig12+8;
      col21=col11+8;
      col22=col12+8;
      lig11div2 = lig11 >> 1;
      col11div2 = col11 >> 1;
      lig12div2 = lig12 >> 1;
      col12div2 = col12 >> 1;
      k=1;

      for(NG=1; NG<=NG_MAX; NG+=INC_NG){

	/* GOB LOOP */

	mbnul=0;
	gob = CIF ? (GOB *)(image_coeff)[NG-1] : (GOB *)(image_coeff)[NG-k++];
	force_value = ForceGobEncoding[NG];
	max_mba = (LIMIT_MBA && (NG==NG_MAX)) ? 12 : 34;

	for(MBA=1; MBA<max_mba; MBA++){
	  register int diff, sum;
	  int lig_11, lig_12, lig_21, lig_22, col_11, col_12, col_21, col_22;
	  int lig_11div2, lig_12div2, col_11div2, col_12div2;

	  /* MACROBLOCK LOOP */

	  col = MBcol[NG][MBA];
	  lig = MBlig[NG][MBA];
	  lig_11=lig+lig11;
	  lig_12=lig+lig12;
	  lig_21=lig+lig21;
	  lig_22=lig+lig22;
	  col_11=col+col11;
	  col_12=col+col12;
	  col_21=col+col21;
	  col_22=col+col22;
	  lig_11div2 = lig_11 >> 1;
	  lig_12div2 = lig_12 >> 1;
	  col_11div2 = col_11 >> 1;
	  col_12div2 = col_12 >> 1;

	  bnul=0;
	  mb = &((*gob)[MBA-1]);

	  /* We choose MB INTRA encoding when a MB has been lost
	  * or more than 30 INTER encoding have been done or
	  * every max_value images (in order to prevent loss packets).
	  */

	  if((mb->last_encoding >= force_value) &&
	     (mb->last_encoding < max_value) && (mb->cpt_inter < 30)){

	    mb->CBC = 0;
	    mb->MODE = INTER;

	    /* B movement detection */
	    
	    sum =  abs((int)new_Y[lig_11][col_11] - 
		       (int)old_Y[lig_11][col_11]);
	    sum += abs((int)new_Y[lig_11][col_12] - 
		       (int)old_Y[lig_11][col_12]);
	    sum += abs((int)new_Y[lig_12][col_11] - 
		       (int)old_Y[lig_12][col_11]);
	    sum += abs((int)new_Y[lig_12][col_12] - 
		       (int)old_Y[lig_12][col_12]);
	    if(sum>threshold){
	      Nblock ++;
	      mb->CBC |= 32;
	    }else
	      bnul ++;
	  
	    sum =  abs((int)new_Y[lig_11][col_21] - 
		       (int)old_Y[lig_11][col_21]);
	    sum += abs((int)new_Y[lig_11][col_22] - 
		       (int)old_Y[lig_11][col_22]);
	    sum += abs((int)new_Y[lig_12][col_21] - 
		       (int)old_Y[lig_12][col_21]);
	    sum += abs((int)new_Y[lig_12][col_22] - 
		       (int)old_Y[lig_12][col_22]);
	    if(sum>threshold){
	      Nblock ++;
	      mb->CBC |= 16;
	    }else
	      bnul ++;

	    sum =  abs((int)new_Y[lig_21][col_11] - 
		       (int)old_Y[lig_21][col_11]);
	    sum += abs((int)new_Y[lig_21][col_12] - 
		       (int)old_Y[lig_21][col_12]);
	    sum += abs((int)new_Y[lig_22][col_11] - 
		       (int)old_Y[lig_22][col_11]);
	    sum += abs((int)new_Y[lig_22][col_12] - 
		       (int)old_Y[lig_22][col_12]);
	    if(sum>threshold){
	      Nblock ++;
	      mb->CBC |= 8;
	    }else
	      bnul ++;

	    sum =  abs((int)new_Y[lig_21][col_21] - 
		       (int)old_Y[lig_21][col_21]);
	    sum += abs((int)new_Y[lig_21][col_22] - 
		       (int)old_Y[lig_21][col_22]);
	    sum += abs((int)new_Y[lig_22][col_21] - 
		       (int)old_Y[lig_22][col_21]);
	    sum += abs((int)new_Y[lig_22][col_22] - 
		       (int)old_Y[lig_22][col_22]);
	    if(sum>threshold){
	      Nblock ++;
	      mb->CBC |= 4;
	    }else
	      bnul ++;

	    if(COLOR){

	      sum =  abs((int)new_Cb[lig_11div2][col_11div2] - 
			 (int)old_Cb[lig_11div2][col_11div2]);
	      sum += abs((int)new_Cb[lig_11div2][col_12div2] - 
		       (int)old_Cb[lig_11div2][col_12div2]);
	      sum += abs((int)new_Cb[lig_12div2][col_11div2] - 
			 (int)old_Cb[lig_12div2][col_11div2]);
	      sum += abs((int)new_Cb[lig_12div2][col_12div2] - 
			 (int)old_Cb[lig_12div2][col_12div2]);
	      if(sum>threshold){
		Nblock ++;
		mb->CBC |= 2;
	      }else
		bnul ++;

	      sum =  abs((int)new_Cb[lig_11div2][col_11div2] - 
			 (int)old_Cb[lig_11div2][col_11div2]);
	      sum += abs((int)new_Cb[lig_11div2][col_12div2] - 
		       (int)old_Cb[lig_11div2][col_12div2]);
	      sum += abs((int)new_Cb[lig_12div2][col_11div2] - 
			 (int)old_Cb[lig_12div2][col_11div2]);
	      sum += abs((int)new_Cb[lig_12div2][col_12div2] - 
			 (int)old_Cb[lig_12div2][col_12div2]);
	      if(sum>threshold){
		Nblock ++;
		mb->CBC |= 1;
	      }else
		bnul ++;
	    }
	    if(bnul == BperMB){
	      mbnul ++;
	      (mb->last_encoding)++;
	      if(mbnul == 33){
		GobEncoded[NG-1]=FALSE;
	      }
	    }else{
	      mb->last_encoding = 0;
	      (mb->cpt_inter)++;
	    }
	  }else{
	    /* INTRA MB encoding */
	    mb->MODE = INTRA;
	    Nblock += BperMB;
	  }
	} /* MACROBLOCK LOOP */
      } /* GOB LOOP */
    } /* if(PARTIAL) */

    /* If there is a change of scene,
    * a full Intra image encoding is done.
    */
    if(PARTIAL){
      if(CIF){
	if(Nblock > nblock_cif_max){
#ifdef DEBUG
	  fprintf(stderr, "Full intra encoding\n");
#endif
	  PARTIAL = FALSE;
	  chaine_codee[0] = first_byte | 0x20;
	  Nblock = 1584;
      }
      }else{
	if(Nblock > nblock_qcif_max){
#ifdef DEBUG
	  fprintf(stderr, "Full intra encoding\n");
#endif
	  PARTIAL = FALSE;
	  chaine_codee[0] = first_byte | 0x20;
	  Nblock = 396;
	}
      }
    }

    k=1;
    for(NG=1; NG<=NG_MAX; NG+=INC_NG){

      /* GOB LOOP */

      if(!PARTIAL || GobEncoded[NG-1]){
	mbnul=0;
	gob = CIF ? (GOB *)(image_coeff)[NG-1] : (GOB *)(image_coeff)[NG-k++]; 
	max_mba = (LIMIT_MBA && (NG==NG_MAX)) ? 12 : 34;

	for(MBA=1; MBA<max_mba; MBA++){

	  /* MACROBLOCK LOOP */

	  col = MBcol[NG][MBA];
	  lig = MBlig[NG][MBA];

	  coldiv2 = col >> 1;
	  ligdiv2 = lig >> 1;
	  mb = &((*gob)[MBA-1]);

	  if(PARTIAL && (mb->MODE == INTER)){

	    max_coeff=0;
	    if((mb->CBC) & 0x20){
	      register int l, c;
	      register int *pt_image = &diff_Y[lig][col];
	      register u_char *pt_new = &new_Y[lig][col];
	      register u_char *pt_old = &old_Y[lig][col];
	      for(l=0; l<7; l++){
		for(c=0; c<8; c++)
		  *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		pt_image += offsetY;
		pt_new += offsetY;
		pt_old += offsetY;
	      }
	      for(c=0; c<8; c++)
		*pt_image++ = (int)*pt_new++ - (int)*pt_old++;
	      max=(*ffct)(&diff_Y[lig][col], mb->P[0], COL_MAX, TRUE, &cmax0);
	      if(max>max_coeff)
		max_coeff=max;
	    }
	    if((mb->CBC) & 0x10){
	      register int l, c;
	      int auxc=col+8;
	      register int *pt_image = &diff_Y[lig][auxc];
	      register u_char *pt_new = &new_Y[lig][auxc];
	      register u_char *pt_old = &old_Y[lig][auxc];
	      for(l=0; l<7; l++){
		for(c=8; c<16; c++)
		  *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		pt_image += offsetY;
		pt_new += offsetY;
		pt_old += offsetY;
	      }
	      for(c=8; c<16; c++)
		*pt_image++ = (int)*pt_new++ - (int)*pt_old++;
	      max=(*ffct)(&diff_Y[lig][auxc], mb->P[1], COL_MAX, TRUE, 
			  &cmax1);
	      if(max>max_coeff)
		max_coeff=max;
	    }
	    if((mb->CBC) & 0x08){
	      register int l, c;
	      int auxl = lig+8;
	      register int *pt_image = &diff_Y[auxl][col];
	      register u_char *pt_new = &new_Y[auxl][col];
	      register u_char *pt_old = &old_Y[auxl][col];
	      for(l=8; l<15; l++){
		for(c=0; c<8; c++)
		  *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		pt_image += offsetY;
		pt_new += offsetY;
		pt_old += offsetY;
	      }
	      for(c=0; c<8; c++)
		*pt_image++ = (int)*pt_new++ - (int)*pt_old++;
	      max=(*ffct)(&diff_Y[auxl][col], mb->P[2], COL_MAX, TRUE, 
			  &cmax2);
	      if(max>max_coeff)
		max_coeff=max;
	    }
	    if((mb->CBC) & 0x04){
	      register int l, c;
	      int auxl = lig+8;
	      int auxc = col+8;
	      register int *pt_image = &diff_Y[auxl][auxc];
	      register u_char *pt_new = &new_Y[auxl][auxc];
	      register u_char *pt_old = &old_Y[auxl][auxc];
	      for(l=8; l<15; l++){
		for(c=8; c<16; c++)
		  *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		pt_image += offsetY;
		pt_new += offsetY;
		pt_old += offsetY;
	      }
	      for(c=8; c<16; c++)
		*pt_image++ = (int)*pt_new++ - (int)*pt_old++;
	      max=(*ffct)(&diff_Y[auxl][auxc], mb->P[3], COL_MAX, TRUE, 
			  &cmax3);
	      if(max>max_coeff)
		max_coeff=max;
	    }
	    if(COLOR){
	      if((mb->CBC) & 0x02){
		register int l, c;
		register int *pt_image = &diff_Cb[ligdiv2][coldiv2];
		register u_char *pt_new = &new_Cb[ligdiv2][coldiv2];
		register u_char *pt_old = &old_Cb[ligdiv2][coldiv2];
		for(l=0; l<7; l++){
		  for(c=0; c<8; c++)
		    *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		  pt_image += offsetC;
		  pt_new += offsetC;
		  pt_old += offsetC;
		}
		for(c=0; c<8; c++)
		  *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		max=(*ffct)(&diff_Cb[ligdiv2][coldiv2], mb->P[4], 
			    COL_MAX_DIV2, TRUE, &cmax4);
		if(max>max_coeff)
		  max_coeff=max;
	      }
	      if((mb->CBC) & 0x01){
		register int l, c;
		register int *pt_image = &diff_Cr[ligdiv2][coldiv2];
		register u_char *pt_new = &new_Cr[ligdiv2][coldiv2];
		register u_char *pt_old = &old_Cr[ligdiv2][coldiv2];
		for(l=0; l<7; l++){
		  for(c=0; c<8; c++)
		    *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		  pt_image += offsetC;
		  pt_new += offsetC;
		  pt_old += offsetC;
		}
		for(c=0; c<8; c++)
		  *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		max=(*ffct)(&diff_Cr[ligdiv2][coldiv2], mb->P[5], 
			    COL_MAX_DIV2, TRUE, &cmax5);
		if(max>max_coeff)
		  max_coeff=max;
	      }
	    }
	  }else{
	    /* INTRA encoding mode */

	    mb->MODE = INTRA;
	    mb->CBC = full_cbc;
	    mb->last_encoding = mb->cpt_inter = 0;
	    max=(*ffct_intra)(&new_Y[lig][col], mb->P[0], COL_MAX, &cmax0);
	    if(max>max_coeff)
	      max_coeff=max;
	    max=(*ffct_intra)(&new_Y[lig][col+8], mb->P[1], COL_MAX, &cmax1);
	    if(max>max_coeff)
	      max_coeff=max;
	    max=(*ffct_intra)(&new_Y[lig+8][col], mb->P[2], COL_MAX, &cmax2);
	    if(max>max_coeff)
	      max_coeff=max;
	    max=(*ffct_intra)(&new_Y[lig+8][col+8], mb->P[3], COL_MAX, &cmax3);
	    if(max>max_coeff)
	      max_coeff=max;
	    if(COLOR){
	      max=(*ffct_intra)(&new_Cb[ligdiv2][coldiv2], mb->P[4], 
				COL_MAX_DIV2, &cmax4);
	      if(max>max_coeff)
		max_coeff=max;
	      max=(*ffct_intra)(&new_Cr[ligdiv2][coldiv2], mb->P[5], 
				COL_MAX_DIV2, &cmax5);
	      if(max>max_coeff)
		max_coeff=max;
	    }
	  }
	  if(mb->CBC){
	    register int n, l, c;

	    quant_mb = (max_coeff >> 8) + 1;
	    if(QUANT_FIX>quant_mb){
	      quant_mb = (PARTIAL ? QUANT_FIX : 8); 
	      /* To avoid large packets sent. */
	    }
	    mb->QUANT = quant_mb;
	    /*
	     *  We have to quantize mb->P[n].
	     */
	    bnul=0;
	    if(quant_mb & 0x01){
	      /* odd quantizer */
	      for(n=0; n<BperMB; n++){
		register int *pt_P = &(mb->P[n][0][0]);
		register int nb_coeff=0;

		if((mb->CBC) & MASK_CBC[n]){
		  cc = *pt_P;
		  for(l=0; l<8; l++)
		    for(c=0; c<8; c++){
		      if(*pt_P > 0){
			if((*pt_P=(*pt_P / quant_mb - 1) / 2))
			  nb_coeff ++;
		      }else{
			if(*pt_P < 0)
			  if((*pt_P=(*pt_P /quant_mb + 1) / 2))
			    nb_coeff ++;
		      }
		      pt_P ++;
		    }
		  if(mb->MODE == INTRA){
		    if((mb->P[n][0][0]==0) && cc)
		      nb_coeff ++;
		    mb->P[n][0][0] = cc;
		  }
		  if(!nb_coeff){
		    bnul ++;
		    (mb->CBC) &= (~MASK_CBC[n]);
		  }
		}else
		  bnul ++;
		mb->nb_coeff[n] = nb_coeff;
	      }
	    }else{
	      /* even quantizer */
	      for(n=0; n<BperMB; n++){
		register int *pt_P = &(mb->P[n][0][0]);
		register int nb_coeff=0;

		if((mb->CBC) & MASK_CBC[n]){
		  cc = *pt_P;
		  for(l=0; l<8; l++)
		    for(c=0; c<8; c++){
		      if(*pt_P > 0){
			if((*pt_P=((*pt_P + 1)/quant_mb - 1)/2 ))
			  nb_coeff ++;
		      }else{
			if(*pt_P < 0)
			  if((*pt_P=((*pt_P - 1)/quant_mb - 1)/2 ))
			    nb_coeff ++;
		      }
		      pt_P ++;
		    }
		  if (mb->MODE == INTRA) {
		      if((!mb->P[n][0][0]) && cc)
			  nb_coeff ++;
		      mb->P[n][0][0] = cc;
		  }
		  if(! nb_coeff){
		      bnul ++;
		      (mb->CBC) &= (~MASK_CBC[n]);
		  }
		}else
		  bnul ++;
		mb->nb_coeff[n] = nb_coeff;
	      }		      
	    }
	    if(bnul == BperMB){
	      mbnul ++;
	      if(mbnul == 33){
		GobEncoded[NG-1] = FALSE;
	      }
	    }
	  }
	  if(bnul != BperMB){
	    /************\
	    * IFCT of MB *
	    \************/
	    int inter = mb->MODE;
	    tab_quant1 = tab_rec1[quant_mb-1];
	    tab_quant2 = tab_rec2[quant_mb-1];
	    
	    if((mb->CBC) & 0x20)
	      ifct_8x8(mb->P[0], tab_quant1, tab_quant2, inter,
		       &(old_Y[lig][col]), cmax0, COL_MAX);
	  
	    if((mb->CBC) & 0x10)
	      ifct_8x8(mb->P[1], tab_quant1, tab_quant2, inter, 
		       &(old_Y[lig][col+8]), cmax1, COL_MAX);
	  
	    if((mb->CBC) & 0x08)
	      ifct_8x8(mb->P[2], tab_quant1, tab_quant2, inter,
		       &(old_Y[lig+8][col]), cmax2, COL_MAX);

	    if((mb->CBC) & 0x04)
	      ifct_8x8(mb->P[3], tab_quant1, tab_quant2, inter,
		       &(old_Y[lig+8][col+8]), cmax3, COL_MAX);
	    if(COLOR){
	      if((mb->CBC) & 0x02)
		ifct_8x8(mb->P[4], tab_quant1, tab_quant2, inter,
			 &(old_Cb[ligdiv2][coldiv2]), cmax4, COL_MAX_DIV2);
	      if((mb->CBC) & 0x01)
		ifct_8x8(mb->P[5], tab_quant1, tab_quant2, inter,
			 &(old_Cr[ligdiv2][coldiv2]), cmax5, COL_MAX_DIV2);
	    }
	  }
	} /* MACROBLOCK LOOP */
      }else
	k ++;
      {
	int last_AM, CBC, quant=QUANT_FIX;
	register int NM, b;

	if(!VIDEO_PACKET_HEADER){
	  chaine_codee[0] = first_byte;
	  chaine_codee[1] = packet_number;
	  chaine_codee[2] = image_number;
	  chaine_codee[3] = (u_char) 0;
	  i=3;
	  j=7;
	  nb_packet ++;
	  VIDEO_PACKET_HEADER=TRUE;
	}

	/*********************\
	* H.261 CODING OF GOB *
	\*********************/

	put_value(&chaine_codee[i],1,16,&i,&j); /* CDGB = 1 */
	put_value(&chaine_codee[i],NG,4,&i,&j); 
	put_value(&chaine_codee[i],quant,5,&i,&j);
	put_bit(chaine_codee,0,i,j); /* ISG */

	if(GobEncoded[NG-1]){
#ifdef STAT
	  fprintf(stderr, "GOB %d, ", NG);
#endif /* STAT */
	  /******************\
          * MACROBLOCK LAYER *
	  \******************/
	  last_AM=0;
	  max_mba = (LIMIT_MBA && (NG==NG_MAX)) ? 12 : 34;

	  for(NM=1; NM<max_mba; NM++){
	      BOOLEAN CBC_IS_CODED;
	    
	    mb = &((*gob)[NM-1]);
	    if(!(mb->CBC)) /* If MB is entirely null, continue ... */
	      continue;
	    if(!chain_forecast(tab_mba,n_mba,(NM-last_AM),chaine_codee,&i,&j)){
	      fprintf(stderr,"Bad MBA value: %d\n", NM-last_AM);
	      exit(-1);
	    }
	    CBC_IS_CODED = (mb->MODE ==INTER);    
	    if(mb->QUANT != quant){
	      quant=mb->QUANT;
	      if(CBC_IS_CODED)
		/*TYPEM=INTRA+CBC+QUANTM*/
		put_value(&chaine_codee[i],1,5,&i,&j);
	      else
		/* TYPEM = INTRA+QUANTM */
		put_value(&chaine_codee[i],1,7,&i,&j);
	      /* Coding of QUANTM */
	      put_value(&chaine_codee[i],quant,5,&i,&j); 
	    }
	    else{
	      if(CBC_IS_CODED)
		/* TYPEM = INTRA+CBC */
		put_bit(chaine_codee,1,i,j); 
	      else
		/* TYPEM = INTRA */
		put_value(&chaine_codee[i],1,4,&i,&j); 
	    }
	    if(CBC_IS_CODED){
	      if(!chain_forecast(tab_cbc, n_cbc, (int)mb->CBC, chaine_codee,
				 &i, &j)){
		fprintf(stderr,"Bad CBC value: %d\n", mb->CBC);
		exit(-1);
	      }
	    }
	    /*************\
	    * BLOCK LAYER *
	    \*************/
	    for(b=0; b<BperMB; b++){
	      if(mb->CBC & MASK_CBC[b]){
		code_bloc_8x8(mb->P[b], chaine_codee, tab_tcoeff, 
			      n_tcoeff, mb->nb_coeff[b], &i, &j, mb->MODE);
	      }
	    }
	    last_AM = NM;
	  } 
	}else{
#ifdef STAT
	  fprintf(stderr, "noGOB %d ", NG);
#endif /* STAT */
#ifdef DEBUG 
	  fprintf(stderr, "GOB %d, ", NG);
#endif
	}

	/* If there is enough data or it is the last GOB,
	* write data on socket
	*/
	if((i > 500) || (NG == NG_MAX)){

	  /* Possible padding */
	  if(7 - j){
	    int max=p_padd[7-j];
	    register int b;

	    for(b=0; b<max; b++)
	      put_value(&chaine_codee[i], 0x0f, 11, &i, &j);
	  }

	  if(NG==NG_MAX){
	    if(STAT_MODE){
	      if(nb_packet>1)
		fprintf(F_loss, "Image %d: packet%d to %d\n", cpt, 
			first_packet, first_packet+nb_packet);
	      else
		fprintf(F_loss, "Image %d: packet%d\n", cpt, first_packet);
	    }
	  }

	  /* Next number packet */
	  packet_number = (packet_number + 1) % 256;

	  if(rate_control == REFRESH_CONTROL){
	    /* Control rate : Privilege quality rather than refreshment. Add
	    *                 if necessary a delay between each packet sent.
	    */
	    static BOOLEAN First=TRUE;
	    static double lasttime;
	    struct timeval realtime;
	    double newtime, deltat;

	    if(First){
	      First=FALSE;
	      gettimeofday(&realtime, (struct timezone *)NULL);
	      lasttime = 1000000.0*realtime.tv_sec + realtime.tv_usec;
	    }else{
	      gettimeofday(&realtime, (struct timezone *)NULL);
	      newtime = 1000000.0*realtime.tv_sec + realtime.tv_usec;
	      deltat = 8000.0*i/rate_max - (newtime - lasttime);
	      if(deltat > 0){
#ifdef DEBUG
		fprintf(stderr, "Control loop: waiting for  %d ms\n", 
		       (int)(deltat/1000));
#endif
		coffee_break((long)deltat);
		lasttime = newtime + deltat;
	      }else
		lasttime = newtime;
	    } 
	  }

	  /* Sending packet */
	  if(i<MAXBSIZE){
	    if((lwrite=sendto(sock, (char *)chaine_codee, i, 0, 
			      (struct sockaddr *) &addr, len_addr)) < 0){
	      perror("sendto: Cannot send data on socket");
	      exit(-1);
	    }
	    length_image += lwrite;
	  }else{
	    fprintf(stderr, "Cannot send %d bytes\n", i);
	  }
#ifdef STAT
	  fprintf(stderr, ":: NUM_PACKET:%d, NB_BYTES:%d\n", packet_number, 
		  lwrite);
	  if(NG==NG_MAX)
	    fprintf(stderr, "Nb(Blocks)= %d\n", Nblock);
#endif /* STAT */
#ifdef DEBUG
	  fprintf(stderr, "GOB %d, NUM_PACKET:%d, NB_BYTES:%d\n", NG, 
		 packet_number, lwrite);
	  if(NG==NG_MAX)
	    fprintf(stderr, "Nb(Blocks)= %d\n", Nblock);
#endif
	  VIDEO_PACKET_HEADER = FALSE;
	}
      } /* Image writing */
    } /* GOB LOOP */
    
    image_number = (image_number+1) % 256;
    nb_lost=NbPacketsLost(sock_recv, &PARTIAL, ForceGobEncoding, image_number,
			  INC_NG);
    cpt ++;

  }




encode_h261_cif4(group, port, ttl, new_Y, new_Cb, new_Cr, image_coeff, 
		 sock_recv, FIRST, COLOR, cif_number, format, rate_control,
		 rate_max, fd)
     char *group, *port;  /* Socket address */
     u_char ttl;          /* ttl value */
     u_char new_Y[][COL_MAX], new_Cb[][COL_MAX_DIV2], new_Cr[][COL_MAX_DIV2];
     IMAGE_CIF image_coeff;
     int sock_recv;       /* socket used for loss messages */
     BOOLEAN FIRST;       /* True if it is the first image */
     BOOLEAN COLOR;       /* True if color encoded */
     int cif_number;      /* select the CIF number in the CIF4 (0,1,2 or 3) */
     int format;          /* VFC_NTSC or VFC_PAL */
     int rate_control;    /* WITHOUT_CONTROL or REFRESH_CONTROL 
			   or QUALITY_CONTROL */
     int rate_max;        /* Maximum bandwidth allowed */
     int fd;              /* Pipe son to father for bandwidth & rate display */


  {
    int lig, col, ligdiv2, coldiv2;
    int NG, MBA;
    GOB *gob;
    MACRO_BLOCK *mb;
    static u_char old_Y[4][LIG_MAX][COL_MAX];
    static u_char old_Cb[4][LIG_MAX_DIV2][COL_MAX_DIV2];
    static u_char old_Cr[4][LIG_MAX_DIV2][COL_MAX_DIV2];
    int diff_Y[LIG_MAX][COL_MAX];
    int diff_Cb[LIG_MAX_DIV2][COL_MAX_DIV2];
    int diff_Cr[LIG_MAX_DIV2][COL_MAX_DIV2];
    int cc, quant_mb;
    int bnul, mbnul;
    int cmax0, cmax1, cmax2, cmax3, cmax4, cmax5;
    int *tab_quant1, *tab_quant2;
    u_char chaine_codee[T_MAX], val;
    int i=0, j=7;
    static int lwrite;
    struct timeval realtime;
    static double inittime, synctime;
    static int RT;
    static int n_padd[8]={4, 1, 6, 3, 0, 5, 2, 7};
    static int p_padd[8]={0, 5, 2, 7, 4, 1, 6, 3};
    static int LIG[16]={0, 2, 0, 1, 3, 1, 3, 2, 0, 3, 2, 1, 3, 0, 2, 1};
    static int COL[16]={0, 2, 3, 1, 0, 2, 3, 1, 2, 1, 3, 0, 2, 1, 0, 3};
    static int MASK_CBC[6]={0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
    static int n_im=0;
    static int lig11, lig12, lig21, lig22, col11, col12, col21, col22;
    static int lig11div2, lig12div2, col11div2, col12div2;
    int Nblock=0;
    int max_coeff, max;
    static int cpt=0;
    static int packet_number=0;
    static int image_number=0;
    static BOOLEAN PARTIAL=FALSE;
    static BOOLEAN ForceGobEncoding[4][13];
    BOOLEAN GobEncoded[12];
    BOOLEAN FFCT8x8=TRUE;
    int nb_lost;
    extern int ffct_4x4(), ffct_8x8();
    extern int ffct_intra_4x4(), ffct_intra_8x8();
    int (*ffct)()=ffct_8x8;
    int (*ffct_intra)()=ffct_intra_8x8;
    int nb_packet=1;
    int first_packet=packet_number;
    int nblock_cif_max=1200;  /* CIF  : if nb(blocks) detected > this value */
    int nblock_qcif_max=300; /* QCIF :    then use INTRA mode              */
    BOOLEAN VIDEO_PACKET_HEADER; /* True if it is set */
    static struct sockaddr_in addr;
    static int sock, len_addr;
    static BOOLEAN first_cif=TRUE;
    static int save_val;
    static int NG_MAX;
    static int BperMB, full_cbc;
    static int max_value=30; /* A block is forced every 30 images */
    int force_value;
    static int offsetY, offsetC; /* Useful to pointers alignment */
    static int threshold=20; /* For motion estimation */
    static int QUANT_FIX=3;
    static int length_image=0;
    static int len_data=11;
    static double fps, bandwidth;
    char data[11];
    double rate;


    /* Header video packet format : 3 bytes
    *  -----------------  00: Video Packet header data.
    *  |0|0|I|P|S|N|N|C|  I : 1 if Full Intra Encoded image.
    *  -----------------  P : 1 if PAL FORMAT.
    *  | packet number |  S : 1 if CIF4, else 0.
    *  -----------------  NN: if CIF4, refers the CIF in the CIF4 (0 to 3). 
    *  | image  number |  C : 1 if color encoded, else 0.
    *  -----------------
    */

    if(first_cif){
      first_cif=FALSE;
      if(COLOR)
	save_val = 1;
      else
	save_val = 0;
#ifdef VIDEOPIX
      if(format == VFC_PAL){
	NG_MAX = 12;
	save_val += 16;
      }else
	NG_MAX = 10;
#else
      NG_MAX = 12;
      save_val += 16;
#endif
      if(COLOR){
	BperMB = 6;
	full_cbc = 63;
      }else{
	BperMB = 4;
	full_cbc = 60;
      }
    }
    val = save_val + (8 + (cif_number<<1));
    if(!PARTIAL && !cif_number)
      chaine_codee[0] = val | 0x20;
    else
      chaine_codee[0] = val;
    chaine_codee[1] = packet_number;
    chaine_codee[2] = image_number;
    chaine_codee[3] = (u_char) 0;
    i = 3;
    VIDEO_PACKET_HEADER = TRUE;

    /*************\
    * IMAGE LAYER *
    \*************/
    gettimeofday(&realtime, (struct timezone *)NULL);
    if(FIRST){
      /* some initializations ... */
      offsetY = COL_MAX - 8;
      offsetC = COL_MAX_DIV2 - 8;
      inittime = realtime.tv_sec + realtime.tv_usec/1000000.0;
      RT=0;
      bzero((char *)image_coeff, sizeof(IMAGE_CIF));
      if(cif_number == 0)
	sock = CreateSendSocket(&addr, &len_addr, port, group, &ttl);
      if(cif_number == 3)
	FIRST = FALSE;
    }else{
      if(cif_number == 0){
	synctime = realtime.tv_sec + realtime.tv_usec/1000000.0;
	RT = ((RT + (int)(33*(synctime-inittime))) & 0x1f);
#ifdef DEBUG
	fprintf(stderr, "Encoding time: %d ms\n", (int)(1000*(synctime-inittime)));
#endif

	if(rate_control == QUALITY_CONTROL){
	  rate = (double)(0.008*length_image) / (synctime-inittime);
	  if(rate > (double)rate_max){
	    if(threshold <= 30)
	      threshold += 5;
	    if(QUANT_FIX <= 11)
	      QUANT_FIX +=2;
#ifdef DEBUG
	    fprintf(stderr, "rate ko: %2.3f, threshold: %d, quant: %d\n", 
		    rate, threshold, QUANT_FIX);
#endif
	  }else{
	    if(threshold > 15)
	      threshold -= 5;
	    if(QUANT_FIX > 3)
	      QUANT_FIX -= 2;
#ifdef DEBUG
	    fprintf(stderr, "rate ok: %2.3f, threshold: %d, quant: %d\n", 
		    rate, threshold, QUANT_FIX);
#endif
	  }
	}

	fps = (1.0 / (synctime-inittime));
	bandwidth = ((double)(0.008*length_image) / (synctime-inittime));
	bzero(data, len_data);
	sprintf(data, "%2.1f", bandwidth);
	sprintf(&data[6], "%2.1f", fps);
	if((lwrite=write(fd, data, len_data)) != len_data){
#ifdef DEBUG
	  perror("encode_h261: Cannot send bandwidth and rate data");
#endif
	}
	length_image = 0;
	inittime = synctime;
      }
      Nblock = 0;
    }
#ifdef DEBUG
    fprintf(stderr, "QUANT: %d, RT: %d\n", QUANT_FIX, RT);
#endif
    put_value(&chaine_codee[i],16,20,&i,&j); /* CDI */
    put_value(&chaine_codee[i],RT,5,&i,&j); /* RT */
    put_value(&chaine_codee[i],4,6,&i,&j); /* TYPEI = CIF */
    put_bit(chaine_codee,0,i,j); /* ISI1 */

    memset(GobEncoded, TRUE, NG_MAX);
    if(PARTIAL){
      if(cif_number == 0){
	lig11=LIG[n_im];
	col11=COL[n_im];
	n_im=(n_im +1)%16;
	lig12=lig11+4;
	col12=col11+4;
	lig21=lig11+8;
	lig22=lig12+8;
	col21=col11+8;
	col22=col12+8;
	lig11div2 = lig11 >> 1;
	col11div2 = col11 >> 1;
	lig12div2 = lig12 >> 1;
	col12div2 = col12 >> 1;
      }

      for(NG=1; NG<=NG_MAX; NG++){

	/* GOB LOOP */

	mbnul=0;
	gob = (GOB *) (image_coeff)[NG-1];
	force_value = ForceGobEncoding[cif_number][NG];

	for(MBA=1; MBA<34; MBA++){
	  register int diff, sum;
	  int lig_11, lig_12, lig_21, lig_22, col_11, col_12, col_21, col_22;
	  int lig_11div2, lig_12div2, col_11div2, col_12div2;

	  /* MACROBLOCK LOOP */

	  col = MBcol[NG][MBA];
	  lig = MBlig[NG][MBA];
	  lig_11=lig+lig11;
	  lig_12=lig+lig12;
	  lig_21=lig+lig21;
	  lig_22=lig+lig22;
	  col_11=col+col11;
	  col_12=col+col12;
	  col_21=col+col21;
	  col_22=col+col22;
	  lig_11div2 = lig_11 >> 1;
	  lig_12div2 = lig_12 >> 1;
	  col_11div2 = col_11 >> 1;
	  col_12div2 = col_12 >> 1;

	  bnul=0;
	  mb = &((*gob)[MBA-1]);

	  /* We choose MB INTRA encoding when a MB has been lost
	  * or more than 30 INTER encoding have been done or
	  * every max_value images (in order to prevent loss packets).
	  */

	  if((mb->last_encoding >= force_value) &&
	     (mb->last_encoding < max_value) && (mb->cpt_inter < 30)){

	    mb->CBC = 0;
	    mb->MODE = INTER;
	    
	    /* B movement detection */

	    sum =  abs((int)new_Y[lig_11][col_11] - 
		       (int)old_Y[cif_number][lig_11][col_11]);
	    sum += abs((int)new_Y[lig_11][col_12] - 
		       (int)old_Y[cif_number][lig_11][col_12]);
	    sum += abs((int)new_Y[lig_12][col_11] - 
		       (int)old_Y[cif_number][lig_12][col_11]);
	    sum += abs((int)new_Y[lig_12][col_12] - 
		       (int)old_Y[cif_number][lig_12][col_12]);
	    if(sum>threshold){
	      Nblock ++;
	      mb->CBC |= 32;
	    }else
	      bnul ++;
	  
	    sum =  abs((int)new_Y[lig_11][col_21] - 
		       (int)old_Y[cif_number][lig_11][col_21]);
	    sum += abs((int)new_Y[lig_11][col_22] - 
		       (int)old_Y[cif_number][lig_11][col_22]);
	    sum += abs((int)new_Y[lig_12][col_21] - 
		       (int)old_Y[cif_number][lig_12][col_21]);
	    sum += abs((int)new_Y[lig_12][col_22] - 
		       (int)old_Y[cif_number][lig_12][col_22]);
	    if(sum>threshold){
	      Nblock ++;
	      mb->CBC |= 16;
	    }else
	      bnul ++;

	    sum =  abs((int)new_Y[lig_21][col_11] - 
		       (int)old_Y[cif_number][lig_21][col_11]);
	    sum += abs((int)new_Y[lig_21][col_12] - 
		       (int)old_Y[cif_number][lig_21][col_12]);
	    sum += abs((int)new_Y[lig_22][col_11] - 
		       (int)old_Y[cif_number][lig_22][col_11]);
	    sum += abs((int)new_Y[lig_22][col_12] - 
		       (int)old_Y[cif_number][lig_22][col_12]);
	    if(sum>threshold){
	      Nblock ++;
	      mb->CBC |= 8;
	    }else
	      bnul ++;

	    sum =  abs((int)new_Y[lig_21][col_21] - 
		       (int)old_Y[cif_number][lig_21][col_21]);
	    sum += abs((int)new_Y[lig_21][col_22] - 
		       (int)old_Y[cif_number][lig_21][col_22]);
	    sum += abs((int)new_Y[lig_22][col_21] - 
		       (int)old_Y[cif_number][lig_22][col_21]);
	    sum += abs((int)new_Y[lig_22][col_22] - 
		       (int)old_Y[cif_number][lig_22][col_22]);
	    if(sum>threshold){
	      Nblock ++;
	      mb->CBC |= 4;
	    }else
	      bnul ++;

	    if(COLOR){

	      sum =  abs((int)new_Cb[lig_11div2][col_11div2] - 
			 (int)old_Cb[cif_number][lig_11div2][col_11div2]);
	      sum += abs((int)new_Cb[lig_11div2][col_12div2] - 
		       (int)old_Cb[cif_number][lig_11div2][col_12div2]);
	      sum += abs((int)new_Cb[lig_12div2][col_11div2] - 
			 (int)old_Cb[cif_number][lig_12div2][col_11div2]);
	      sum += abs((int)new_Cb[lig_12div2][col_12div2] - 
			 (int)old_Cb[cif_number][lig_12div2][col_12div2]);
	      if(sum>threshold){
		Nblock ++;
		mb->CBC |= 2;
	      }else
		bnul ++;

	      sum =  abs((int)new_Cr[lig_11div2][col_11div2] - 
			 (int)old_Cr[cif_number][lig_11div2][col_11div2]);
	      sum += abs((int)new_Cr[lig_11div2][col_12div2] - 
			 (int)old_Cr[cif_number][lig_11div2][col_12div2]);
	      sum += abs((int)new_Cr[lig_12div2][col_11div2] - 
			 (int)old_Cr[cif_number][lig_12div2][col_11div2]);
	      sum += abs((int)new_Cr[lig_12div2][col_12div2] - 
			 (int)old_Cr[cif_number][lig_12div2][col_12div2]);
	      if(sum>threshold){
		Nblock ++;
		mb->CBC |= 1;
	      }else
		bnul ++;
	    }
	    if(bnul == BperMB){
	      mbnul ++;
	      (mb->last_encoding)++;
	      if(mbnul == 33){
		GobEncoded[NG-1] = FALSE;
	      }
	    }else{
	      mb->last_encoding = 0;
	      (mb->cpt_inter) ++;
	    }
	  }else{
	    /* INTRA MB encoding */
	    mb->MODE = INTRA;
	    Nblock += BperMB;
	  }
	} /* MACROBLOCK LOOP */
      } /* GOB LOOP */
    } /* if(PARTIAL) */

    /* If there is a change of scene,
    * a full Intra image encoding is done.
    */

    if(Nblock > nblock_cif_max){
      PARTIAL = FALSE;
      chaine_codee[0] = val | 0x20;
    }

    for(NG=1; NG<=NG_MAX; NG++){

      /* GOB LOOP */

      if(!PARTIAL || GobEncoded[NG-1]){
	mbnul=0;
	gob = (GOB *) (image_coeff)[NG-1];

	for(MBA=1; MBA<34; MBA++){

	  /* MACROBLOCK LOOP */

	  col = MBcol[NG][MBA];
	  lig = MBlig[NG][MBA];

	  coldiv2 = col >> 1;
	  ligdiv2 = lig >> 1;

	  mb = &((*gob)[MBA-1]);
	  if(PARTIAL && (mb->MODE == INTER)){

	    max_coeff=0;
	    if((mb->CBC) & 0x20){
	      register int l, c;
	      register int *pt_image = &diff_Y[lig][col];
	      register u_char *pt_new = &new_Y[lig][col];
	      register u_char *pt_old = &old_Y[cif_number][lig][col];
	      for(l=0; l<7; l++){
		for(c=0; c<8; c++)
		  *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		pt_image += offsetY;
		pt_new += offsetY;
		pt_old += offsetY;
	      }
	      for(c=0; c<8; c++)
		*pt_image++ = (int)*pt_new++ - (int)*pt_old++;
	      max=(*ffct)(&diff_Y[lig][col], mb->P[0], COL_MAX, TRUE, &cmax0);
	      if(max>max_coeff)
		max_coeff=max;
	    }
	    if((mb->CBC) & 0x10){
	      register int l, c;
	      int auxc=col+8;
	      register int *pt_image = &diff_Y[lig][auxc];
	      register u_char *pt_new = &new_Y[lig][auxc];
	      register u_char *pt_old = &old_Y[cif_number][lig][auxc];
	      for(l=0; l<7; l++){
		for(c=8; c<16; c++)
		  *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		pt_image += offsetY;
		pt_new += offsetY;
		pt_old += offsetY;
	      }
	      for(c=8; c<16; c++)
		*pt_image++ = (int)*pt_new++ - (int)*pt_old++;
	      max=(*ffct)(&diff_Y[lig][auxc], mb->P[1], COL_MAX, TRUE, 
			  &cmax1);
	      if(max>max_coeff)
		max_coeff=max;
	    }
	    if((mb->CBC) & 0x08){
	      register int l, c;
	      int auxl = lig+8;
	      register int *pt_image = &diff_Y[auxl][col];
	      register u_char *pt_new = &new_Y[auxl][col];
	      register u_char *pt_old = &old_Y[cif_number][auxl][col];
	      for(l=8; l<15; l++){
		for(c=0; c<8; c++)
		  *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		pt_image += offsetY;
		pt_new += offsetY;
		pt_old += offsetY;
	      }
	      for(c=0; c<8; c++)
		*pt_image++ = (int)*pt_new++ - (int)*pt_old++;
	      max=(*ffct)(&diff_Y[auxl][col], mb->P[2], COL_MAX, TRUE, 
			  &cmax2);
	      if(max>max_coeff)
		max_coeff=max;
	    }
	    if((mb->CBC) & 0x04){
	      register int l, c;
	      int auxl = lig+8;
	      int auxc = col+8;
	      register int *pt_image = &diff_Y[auxl][auxc];
	      register u_char *pt_new = &new_Y[auxl][auxc];
	      register u_char *pt_old = &old_Y[cif_number][auxl][auxc];
	      for(l=8; l<15; l++){
		for(c=8; c<16; c++)
		  *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		pt_image += offsetY;
		pt_new += offsetY;
		pt_old += offsetY;
	      }
	      for(c=8; c<16; c++)
		*pt_image++ = (int)*pt_new++ - (int)*pt_old++;
	      max=(*ffct)(&diff_Y[auxl][auxc], mb->P[3], COL_MAX, TRUE, 
			  &cmax3);
	      if(max>max_coeff)
		max_coeff=max;
	    }
	    if(COLOR){
	      if((mb->CBC) & 0x02){
		register int l, c;
		register int *pt_image = &diff_Cb[ligdiv2][coldiv2];
		register u_char *pt_new = &new_Cb[ligdiv2][coldiv2];
		register u_char *pt_old = 
		  &old_Cb[cif_number][ligdiv2][coldiv2];
		for(l=0; l<7; l++){
		  for(c=0; c<8; c++)
		    *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		  pt_image += offsetC;
		  pt_new += offsetC;
		  pt_old += offsetC;
		}
		for(c=0; c<8; c++)
		  *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		max=(*ffct)(&diff_Cb[ligdiv2][coldiv2], mb->P[4], 
			    COL_MAX_DIV2, TRUE, &cmax4);
		if(max>max_coeff)
		  max_coeff=max;
	      } 
	      if((mb->CBC) & 0x01){
		register int l, c;
		register int *pt_image = &diff_Cr[ligdiv2][coldiv2];
		register u_char *pt_new = &new_Cr[ligdiv2][coldiv2];
		register u_char *pt_old = 
		  &old_Cr[cif_number][ligdiv2][coldiv2];
		for(l=0; l<7; l++){
		  for(c=0; c<8; c++)
		    *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		  pt_image += offsetC;
		  pt_new += offsetC;
		  pt_old += offsetC;
		}
		for(c=0; c<8; c++)
		  *pt_image++ = (int)*pt_new++ - (int)*pt_old++;
		max=(*ffct)(&diff_Cr[ligdiv2][coldiv2], mb->P[5], 
			    COL_MAX_DIV2, TRUE, &cmax5);
		if(max>max_coeff)
		  max_coeff=max;
	      }
	    }
	  }else{
	    /* INTRA encoding mode */
	    mb->MODE = INTRA;
	    mb->CBC = full_cbc;
	    mb->last_encoding = mb->cpt_inter = 0;
	    max=(*ffct_intra)(&new_Y[lig][col], mb->P[0], COL_MAX, &cmax0);
	    if(max>max_coeff)
	      max_coeff=max;
	    max=(*ffct_intra)(&new_Y[lig][col+8], mb->P[1], COL_MAX, &cmax1);
	    if(max>max_coeff)
	      max_coeff=max;
	    max=(*ffct_intra)(&new_Y[lig+8][col], mb->P[2], COL_MAX, &cmax2);
	    if(max>max_coeff)
	      max_coeff=max;
	    max=(*ffct_intra)(&new_Y[lig+8][col+8], mb->P[3], COL_MAX, &cmax3);
	    if(max>max_coeff)
	      max_coeff=max;
	    if(COLOR){
	      max=(*ffct_intra)(&new_Cb[ligdiv2][coldiv2], mb->P[4], 
				COL_MAX_DIV2, &cmax4);
	      if(max>max_coeff)
		max_coeff=max;
	      max=(*ffct_intra)(&new_Cr[ligdiv2][coldiv2], mb->P[5], 
				COL_MAX_DIV2, &cmax5);
	      if(max>max_coeff)
		max_coeff=max;
	    }
	  }
	  if(mb->CBC){
	    register int n, l, c;

	    quant_mb = max_coeff/256 + 1;
	    if(QUANT_FIX>quant_mb)
	      quant_mb=QUANT_FIX;
	    mb->QUANT = quant_mb;
	    /*
	     *  We have to quantize mb->P[n].
	     */
	    bnul=0;
	    if(quant_mb & 0x01){
	      /* odd quantizer */
	      for(n=0; n<BperMB; n++){
		register int *pt_P = &(mb->P[n][0][0]);
		register int nb_coeff=0;

		if((mb->CBC) & MASK_CBC[n]){
		  cc = *pt_P;
		  for(l=0; l<8; l++)
		    for(c=0; c<8; c++){
		      if(*pt_P > 0){
			if((*pt_P=(*pt_P / quant_mb - 1)/2))
			  nb_coeff ++;
		      }else{
			if(*pt_P < 0)
			  if((*pt_P=(*pt_P /quant_mb + 1)/2))
			    nb_coeff ++;
		      }
		      pt_P ++;
		    }
		  if(mb->MODE == INTRA){
		    if((mb->P[n][0][0]==0) && cc)
		      nb_coeff ++;
		    mb->P[n][0][0] = cc;
		  }
		  if(!nb_coeff){
		    bnul ++;
		    (mb->CBC) &= (~MASK_CBC[n]);
		  }
		}else
		  bnul ++;
		mb->nb_coeff[n] = nb_coeff;
	      }
	    }else{
	      /* even quantizer */
	      for(n=0; n<BperMB; n++){
		register int *pt_P = &(mb->P[n][0][0]);
		register int nb_coeff=0;

		if((mb->CBC) & MASK_CBC[n]){
		  cc = *pt_P;
		  for(l=0; l<8; l++)
		    for(c=0; c<8; c++){
		      if(*pt_P > 0){
			if((*pt_P=((*pt_P + 1)/quant_mb - 1)/2))
			  nb_coeff ++;
		      }else{
			if(*pt_P < 0)
			  if((*pt_P=((*pt_P - 1)/quant_mb - 1)/2))
			    nb_coeff ++;
		      }
		      pt_P ++;
		    }
		  if (mb->MODE == INTRA) {
		      if((!mb->P[n][0][0]) && cc)
			  nb_coeff ++;
		      mb->P[n][0][0] = cc;
		  }
		  if(! nb_coeff){
		      bnul ++;
		      (mb->CBC) &= (~MASK_CBC[n]);
		  }
	      }else
		  bnul ++;
		mb->nb_coeff[n] = nb_coeff;
	      }		      
	    }
	    if(bnul == BperMB){
	      mbnul ++;
	      if(mbnul == 33){
		GobEncoded[NG-1]=FALSE;
	      }
	    }
	  }
	  if(bnul != BperMB){
	    /************\
	    * IFCT of MB *
	    \************/
	    int inter = mb->MODE;
	    tab_quant1 = tab_rec1[quant_mb-1];
	    tab_quant2 = tab_rec2[quant_mb-1];
	    
	    if((mb->CBC) & 0x20)
	      ifct_8x8(mb->P[0], tab_quant1, tab_quant2, inter,
		       &(old_Y[cif_number][lig][col]), cmax0, COL_MAX);
	  
	    if((mb->CBC) & 0x10)
	      ifct_8x8(mb->P[1], tab_quant1, tab_quant2, inter, 
		       &(old_Y[cif_number][lig][col+8]), cmax1, COL_MAX);
	  
	    if((mb->CBC) & 0x08)
	      ifct_8x8(mb->P[2], tab_quant1, tab_quant2, inter,
		       &(old_Y[cif_number][lig+8][col]), cmax2, COL_MAX);

	    if((mb->CBC) & 0x04)
	      ifct_8x8(mb->P[3], tab_quant1, tab_quant2, inter,
		       &(old_Y[cif_number][lig+8][col+8]), cmax3, COL_MAX);
	    if(COLOR){
	      if((mb->CBC) & 0x02)
		ifct_8x8(mb->P[4], tab_quant1, tab_quant2, inter,
			 &(old_Cb[cif_number][ligdiv2][coldiv2]), cmax4, 
			 COL_MAX_DIV2);
	      if((mb->CBC) & 0x01)
		ifct_8x8(mb->P[5], tab_quant1, tab_quant2, inter,
			 &(old_Cr[cif_number][ligdiv2][coldiv2]), cmax5, 
			 COL_MAX_DIV2);
	    }
	  }
	} /* MACROBLOCK LOOP */
      }
      {
	int last_AM, CBC, quant=QUANT_FIX;
	register int NM, b;

	if(!VIDEO_PACKET_HEADER){
	  chaine_codee[0] = val;
	  chaine_codee[1] = packet_number;
	  chaine_codee[2] = image_number;
	  chaine_codee[3] = (u_char) 0;
	  i=3;
	  j=7;
	  nb_packet ++;
	  VIDEO_PACKET_HEADER = TRUE;
	}

	/*********************\
	* H.261 CODING OF GOB *
	\*********************/

	put_value(&chaine_codee[i],1,16,&i,&j); /* CDGB = 1 */
	put_value(&chaine_codee[i],NG,4,&i,&j); 
	put_value(&chaine_codee[i],quant,5,&i,&j);
	put_bit(chaine_codee,0,i,j); /* ISG */

	if(GobEncoded[NG-1]){
#ifdef STAT
	  fprintf(stderr, "GOB %d, ", NG);
#endif 

	  /******************\
          * MACROBLOCK LAYER *
	  \******************/

	  last_AM=0;
	  for(NM=1; NM<=33; NM++){
	      BOOLEAN CBC_IS_CODED;
	    
	    mb = &((*gob)[NM-1]);
	    if(!(mb->CBC)) /* If MB is entirely null, continue ... */
	      continue;
	    if(!chain_forecast(tab_mba,n_mba,(NM-last_AM),chaine_codee,&i,&j)){
	      fprintf(stderr,"Bad MBA value: %d\n", NM-last_AM);
	      exit(-1);
	    }
	    CBC_IS_CODED = (mb->MODE ==INTER);    
	    if(mb->QUANT != quant){
	      quant=mb->QUANT;
	      if(CBC_IS_CODED)
		/*TYPEM=INTRA+CBC+QUANTM*/
		put_value(&chaine_codee[i],1,5,&i,&j);
	      else
		/* TYPEM = INTRA+QUANTM */
		put_value(&chaine_codee[i],1,7,&i,&j);
	      /* Coding of QUANTM */
	      put_value(&chaine_codee[i],quant,5,&i,&j); 
	    }
	    else{
	      if(CBC_IS_CODED)
		/* TYPEM = INTRA+CBC */
		put_bit(chaine_codee,1,i,j); 
	      else
		/* TYPEM = INTRA */
		put_value(&chaine_codee[i],1,4,&i,&j); 
	    }
	    if(CBC_IS_CODED){
	      if(!chain_forecast(tab_cbc, n_cbc, (int)mb->CBC, chaine_codee,
				 &i, &j)){
		fprintf(stderr,"Bad CBC value: %d\n", mb->CBC);
		exit(-1);
	      }
	    }
	    /*************\
	    * BLOCK LAYER *
	    \*************/
	    for(b=0; b<BperMB; b++)
	      if(mb->CBC & MASK_CBC[b])
		code_bloc_8x8(mb->P[b], chaine_codee, tab_tcoeff, 
			      n_tcoeff, mb->nb_coeff[b], &i, &j, mb->MODE);
	    last_AM = NM;
	  } 
	}else{
#ifdef STAT
	  fprintf(stderr, "noGOB %d ", NG);
#endif /* STAT */
#ifdef DEBUG 
	  fprintf(stderr, "GOB %d, ", NG);
#endif
	}

	/* If there is enough data or it is the last GOB,
	* write data on socket.
	*/

	if((i > 500) || (NG == NG_MAX)){
	    
	  /* Possible padding */
	  if(7 - j){
	    int max=p_padd[7-j];
	    register int b, c;

	    for(b=0; b<max; b++)
	      put_value(&chaine_codee[i], 0x0f, 11, &i, &j);
	  }

	  if(NG==NG_MAX){
	    if(STAT_MODE){
	      if(nb_packet>1)
		fprintf(F_loss, "Image %d: packet%d to %d\n", cpt, 
			first_packet, (first_packet+nb_packet)%256);
	      else
		fprintf(F_loss, "Image %d: packet%d\n", cpt, first_packet);
	    }
	  }

	  /* Next number packet */
	  packet_number = (packet_number + 1) % 256;

	  if(rate_control == REFRESH_CONTROL){
	    /* Control rate : Privilege quality rather than refreshment.
	    *  Add if necessary a delay between each packet sent.
	    */
	    static BOOLEAN First=TRUE;
	    static double lasttime;
	    struct timeval realtime;
	    double newtime, deltat;
	    int n;

	    if(First){
	      First=FALSE;
	      gettimeofday(&realtime, (struct timezone *)NULL);
	      lasttime = 1000000.0*realtime.tv_sec + realtime.tv_usec;
	    }else{
	      gettimeofday(&realtime, (struct timezone *)NULL);
	      newtime = 1000000.0*realtime.tv_sec + realtime.tv_usec;
	      deltat = 8000.0*i/rate_max - (newtime - lasttime);
	      if(deltat > 0){
#ifdef DEBUG
		fprintf(stderr, "Waiting for %d ms\n", (int)(deltat/1000));
#endif
		coffee_break((long)deltat);
		lasttime = newtime + deltat;
	      }else
		lasttime = newtime;
	    } 
	  }
  
	  /* Sending packet */
	  if(i<MAXBSIZE){
	    if((lwrite=sendto(sock, (char *)chaine_codee, i, 0, 
			      (struct sockaddr *) &addr, len_addr)) < 0){
	      perror("sendto: Cannot send data on socket");
	      exit(-1);
	    }
	    length_image += lwrite;
	  }else{
	    fprintf(stderr, "Cannot send %d bytes\n", i);
	  }
#ifdef STAT
	  fprintf(stderr, ":: NUM_PACKET:%d, NB_BYTES:%d\n", packet_number, 
		  lwrite);
	  if(NG==NG_MAX)
	    fprintf(stderr, "Nb(Blocks)= %d\n", Nblock);
#endif /* STAT */
#ifdef DEBUG
	  fprintf(stderr, "GOB %d, NUM_PACKET:%d, NB_BYTES:%d\n", NG, 
		 packet_number, lwrite);
	  if(NG==NG_MAX)
	    fprintf(stderr, "Nb(Blocks)= %d\n", Nblock);
#endif
	  VIDEO_PACKET_HEADER = FALSE;
	}
      } /* Image writing */
    } /* GOB LOOP */

    if(cif_number == 3){
      image_number = (image_number + 1) % 256;
      nb_lost=NbPacketsLost4(sock_recv, &PARTIAL, ForceGobEncoding, 
			     image_number);
    }
    cpt ++;

  }




int chain_forecast(tab_code, n_code, result, chain, i, j)
    /*
    *  Look for associated code to value "result" in the sorted array 
    *  "tab_code". If this code exists then :
    *    - append it to chain (big endian).
    *    - return 1
    *  Else
    *    - return 0.
    */

    mot_code *tab_code;
    int n_code;
    int result;
    unsigned char *chain;
    int *i; /* current byte */
    int *j; /* next bit to code */

  {
    register int d, g, m, aux;
    register int ii = *i, jj = *j;
    u_char *chain_ptr;

    d = n_code - 1;
    g = 0;
    m = (g+d) >> 3;
    chain_ptr = &chain[ii];
    do{
      aux = tab_code[m].result;
      if(result < aux)
	  d = m - 1;
      else if(result > aux)
	  g = m + 1;
      else {
	  register int bit;
	  register char zero='0';
	  register u_char *tab_ptr;
	  tab_ptr = (u_char *) tab_code[m].chain;
	  while((bit = *tab_ptr++)){
	      put_bit_ptr(chain_ptr,(bit ^ zero),ii,jj);
	  }
	  *i=ii; *j=jj;
	  return(1);
      }
      m = (g+d) >> 1;            
    }while(g <= d);
    return(0);
  }
/*
 {
    register int d, g, m, aux;
    register int ii = *i, jj = *j;
    u_char *chain_ptr;

    d = n_code - 1;
    g = 0;
    chain_ptr = &chain[ii];
    do{
      m = (g+d) >> 1;
      aux = tab_code[m].result;
      if(result <= aux)
	  d = m - 1;
      if(result >= aux)
	  g = m + 1;
    }while(g <= d);
    if(aux == result){
      register int k=0;
      do{
	register int bit;

	bit = (tab_code[m].chain[k] == '0' ? 0 : 1);
	put_bit_ptr(chain_ptr,bit,ii,jj);
	k ++;
      }while(tab_code[m].chain[k] != '\0');
      *i=ii; *j=jj;
      return(1);
    }
    *i=ii; *j=jj;
    return(0);
  }
*/


code_bloc_8x8(bloc, chaine_codee, tab_code, n_code, nb_coeff, i, j, inter)
 
    /*
    *  H.261 encoding of a 8x8 block of DCT coefficients.
    */

    int bloc[][8];
    unsigned char *chaine_codee;
    mot_code *tab_code;
    int n_code, nb_coeff;
    int *i, *j;
    int inter; /* Boolean: True if INTER mode*/
      
  {
    unsigned char run = 0; 
    register int level; 
    register int val, a, k;
    int plus, N=0;
    int first_coeff = 1; /* Boolean */
    int ii = *i, jj = *j;
    static unsigned char *zig_ptr, *zag_ptr;
    u_char *chain_ptr;
    
    zig_ptr = &ZIG[0];
    zag_ptr = &ZAG[0];
    
    for(a=0; a<64; a++){
      if(N==nb_coeff)
	  break;
      if(!(val=bloc[*zig_ptr++][*zag_ptr++])){
	run ++;
	continue;
      }else
	  N ++;
      if(val>0){
	level = val;
	plus = 1;
      }else{
	level = -val;
	plus = 0;
      }
      val = (run<<4) + level; /* run is encoded with 5 bits (to the left)
			      *  and level with 4 bits (to the right)
			      */
      if(first_coeff){
	first_coeff = 0;
	if(inter){
	  if((level == 1)&&(!run)){
	    /* Peculiar case : First coeff is 1 */
	    put_bit(chaine_codee,1,ii,jj);
	    if(plus)
	      put_bit(chaine_codee,0,ii,jj);
	    else
	      put_bit(chaine_codee,1,ii,jj);
	    continue; 
	  }
	}else{
	  if(a != 0)
	    fprintf(stderr, "CC INTRA = 0 !!!\n");
	  /* INTRA mode and CC --> encoding with an uniform quantizer */
	  if(level == 128)
	      level = 255;
	  put_value(&chaine_codee[ii], level, 8, &ii, &jj);
	  continue;
	}
      }
      if(level<=15) {
	  if(chain_forecast(tab_code,n_code,val,chaine_codee,&ii,&jj)){
	    /* Sign encoding */
	    if(plus)
		put_bit(chaine_codee,0,ii,jj);
	    else
		put_bit(chaine_codee,1,ii,jj);
	  run = 0;
	  continue;
	  }
      }
      /*
      *  This code is not a Variable Length Code, so we have to encode it 
      *  with 20 bits.
      *  - 6 bits for ESCAPE (000001)
      *  - 6 bits for RUN
      *  - 8 bits for LEVEL
      */

      put_value(&chaine_codee[ii], 1, 6, &ii, &jj);
      put_value(&chaine_codee[ii], run, 6, &ii, &jj);

      if(plus)
	  put_value(&chaine_codee[ii],level, 8, &ii, &jj);
      else{
	  put_bit(chaine_codee,1,ii,jj);
	  level = 128 - level;
	  put_value(&chaine_codee[ii],level, 7, &ii, &jj);
      }
      run = 0;
    }
    /* EOB encoding (10) */
    put_value(&chaine_codee[ii], 2, 2, &ii, &jj);
    *i=ii; *j=jj;
  }
	
	
/* This routine sets bits in the chain eight at a time */
int put_value(chain_ptr, value, n_bits, i, j) 
     unsigned char *chain_ptr;
     int value, n_bits;
     int *i, *j;
{
  while ( *j <= (n_bits - 1)){
    *chain_ptr++ |= ( value >> ((n_bits - 1 ) - *j));
    n_bits -= (*j + 1);
    *chain_ptr = 0;
    *j = 7;
    *i += 1;
  }
  *chain_ptr |=(value << ( *j - (n_bits - 1)));
  *j -= n_bits;
}
