/**************************************************************************\
*          Copyright (c) 1992 INRIA Sophia Antipolis, FRANCE.              *
*                                                                          *
* Permission to use, copy, modify, and distribute this material for any    *
* purpose and without fee is hereby granted, provided that the above       *
* copyright notice and this permission notice appear in all copies.        *
* WE MAKE NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS     *
* MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS   *
* OR IMPLIED WARRANTIES.                                                   *
\**************************************************************************/
/**************************************************************************\
*                                                                          *
*  File    : video_decoder.c                                               *
*  Date    : October 1992                                                  *
*  Author  : Thierry Turletti                                              *
*--------------------------------------------------------------------------*
*  Description :  H.261 decoding procedures.                               *
*                                                                          *
\**************************************************************************/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/time.h>
#include "h261.h"
#include "protocol.h"
#include "huffman.h"
#include "quantizer.h"

#define ESCAPE        0
#define EOB          -1
#define L_Y         352
#define L_Y_DIV2    176
#define T_READ_MAX 4096
#define CDI          -1 /* CDI decoding */
#define PADDING       0 /* Padding decoding (11 bits) */


#define get_bit(octet,i,j) \
    (j==7 ? (j=0, ((i)+1)<lmax ? octet[(i)++]&0x01 : (vali=octet[i], \
			 lmax=ReadBuffer(sock, octet, &i), vali&0x01)) \
          : (octet[(i)]>>(7-(j)++))&0x01)

/*----------------*\
* Global variables *
\*----------------*/


static unsigned char ZIG[] =
  { /* Raw coefficient block in ZIGZAG scan order */
   0,0,1,2,1,0,0,1,2,3,4,3,2,1,0,0,1,2,3,4,5,6,5,4,3,2,1,0,0,1,2,3,
   4,5,6,7,7,6,5,4,3,2,1,2,3,4,5,6,7,7,6,5,4,3,4,5,6,7,7,6,5,6,7,7
  };

static unsigned char ZAG[] =
  { /* Column coefficient block in ZIGZAG scan order */
   0,1,0,0,1,2,3,2,1,0,0,1,2,3,4,5,4,3,2,1,0,0,1,2,3,4,5,6,7,6,5,4,
   3,2,1,0,1,2,3,4,5,6,7,7,6,5,4,3,2,3,4,5,6,7,7,6,5,4,5,6,7,7,6,7
  };

int sock, lmax, vali, fd_pipe;
char host_coder[50];
u_long host_sin_addr;
Display *dpy;
XImage *ximage;
GC gc;
Window win;
XColor colors[256];
u_char map_lut[256]; 
int N_LUTH=256; 
int depth;

BOOLEAN COLOR_ENCODED;
BOOLEAN SUPER_CIF, PAL_FORMAT;
BOOLEAN FULL_INTRA_REQUEST=TRUE;
BOOLEAN FULL_INTRA; /* True if Full INTRA encoded packet */
int cif_number;
int packet_number, image_number;

int NB_IMAGE=0;
double inittime, synctime, TT;
struct timeval realtime;


debug_block(block)
     int block[][8];
{
  register int l, c;

  for(l=0; l<8; l++){
    fprintf(stderr, "\n");
    for(c=0; c<8; c++)
      fprintf(stderr, "%d ", block[l][c]);
  }
  fprintf(stderr, "\n");
}


RetrieveBitError(chaine_codee, i, j)
     u_char *chaine_codee;
     int *i, *j;
{
  int result, new_x, new_bit, x;

  /* This procedure is called if a bit has been erroned.
  *  In such a case, a full INTRA request is sent to the coder,
  *  and the decoding process continue with the next GOB received.
  *  After this call, (i,j) refer to the GOB number NG.
  */

  SendFullIntraRequest(host_coder);
  *j = 7;
  *i = lmax; /* So, ReadBuffer will be called at the next get_bit call. */
  new_x = 0;
  do{
    do{
      x = new_x;
      new_bit = get_bit(chaine_codee, *i, *j);
      new_x = state_mba[x].bit[new_bit];
    }while(new_x != 0);
    result = state_mba[x].result[new_bit];
    if(result == PADDING)
      continue;
  }while(result != CDI);
}      


decode_bloc_8x8(chaine_codee, bloc, state, i, j, colmax, inter)
    unsigned char *chaine_codee;
    int bloc [][8];
    etat *state;
    int *i, *j;
    int *colmax; /* bloc[][colmax+1] = 0 */
    BOOLEAN inter; /* True if INTER mode */

  {
    int x, new_x; /* previous and new state */
    int run, level, result;
    int temp, new_bit, negatif;
    int col, maxcol=0;
    register int l, k, jj;
    int ii;

    k = 0; /* Next coefficient block. */
    ii = *i;
    jj = *j;
    bzero((char *)bloc, 256);
    if(inter){
      if((chaine_codee[ii]>>(7 - jj))&0x01){
	/* Special case: First coeff starts with 1
	*  So, run=0 and abs(level)=1.
	*/
        temp = get_bit(chaine_codee,ii,jj);
	if((col=ZAG[k]) > maxcol)
	    maxcol=col;
        if(get_bit(chaine_codee,ii,jj)){
	  /* level = -1 */
  	  bloc[ZIG[k]][col] = -1;
	  k ++;
        }else{
	  /* level = +1 */
	  bloc[ZIG[k]][col] = 1;
	  k ++;
        }
      }
    }else{ /* INTRA mode and CC */
      level = get_bit(chaine_codee,ii,jj);
      for(l=1; l<8; l++)
	  level = (level<<1) + get_bit(chaine_codee,ii,jj);
      level = (level==255 ? 128 : level);
      if((col=ZAG[k]) > maxcol)
	  maxcol=col;
      bloc[ZIG[k]][col] = level;
      k ++;
    }
    do{
      new_x = 0;
      do{
	x = new_x;
	new_bit = get_bit(chaine_codee,ii,jj);
	new_x = state[x].bit[new_bit];
      }while(new_x != 0);
      if(k>64)
	fprintf(stderr, "k = %d !!!\n", k);
      if((result=state[x].result[new_bit]) > 0){
	/* It is a Variable Length Code. */
	run = result >> 4;
	level = result & 0x0f;
	k += run;
	if((col=ZAG[k]) > maxcol)
	    maxcol=col;
	/* Level's sign. */
	if(get_bit(chaine_codee,ii,jj))
	    bloc[ZIG[k]][col] = -level;
	else
	    bloc[ZIG[k]][col] = level;
	k ++;
	continue;
      }
      if(result == ESCAPE){
	/* It is a fixed length code -> 20 bits. */
	/* 6 bits(ESCAPE) + 6 bits(RUN) + 8 bits(LEVEL) */
	run = 0;
	level = 0;
	for(l=0; l<6; l++)
	    run = (run << 1) + get_bit(chaine_codee,ii,jj);
	negatif = get_bit(chaine_codee,ii,jj);
	for(l=0; l<7; l++)
	    level = (level << 1) + get_bit(chaine_codee,ii,jj);
	if(negatif)
	    level -= 128;
	k += run;
	if((col=ZAG[k]) > maxcol)
	    maxcol=col;
        bloc[ZIG[k]][col] = level;
        k ++;
        continue;
      }
      if(result == EOB)
	  break;

    }while(1);
    *colmax=maxcol;
    *i = ii;
    *j = jj;
  }



int ReadBuffer(sock, buf, i)
    int sock;
    u_char *buf;
    int *i;

  {
    struct sockaddr_in addr_from;
    int lr, mask, type;
    u_char memo[256];
    static int fromlen;
    static int mask0, mask_pipe, fmax;
    static int packet_expected;
    static BOOLEAN first=TRUE;
    static int nb_requests=0;

    if(first){
      mask_pipe = 1 << fd_pipe;
      mask0 = mask_pipe | (1 << sock);
      fmax=(fd_pipe>sock ? fd_pipe+1 : sock+1);
      fromlen = sizeof(addr_from);
    }

    do{
      mask=mask0;
      if(select(fmax, (fd_set *)&mask, (fd_set *)0, (fd_set *)0, 
		(struct timeval *)0) <= 0){
	perror("VideoDecode'ReadBuffer'select");
	exit(1);
      }
      if(mask & mask_pipe){
	if((lr=read(fd_pipe, memo, N_LUTH)) != N_LUTH){
	  gettimeofday(&realtime, (struct timezone *)NULL);
	  synctime = realtime.tv_sec + realtime.tv_usec/1000000.0;
	  TT = (synctime-inittime);
#ifdef DEBUG
	  fprintf(stderr, "NB_IMAGES: %d, decoding time: %f nb(images/s): %2.4f\n",
		 NB_IMAGE, TT, (double)(NB_IMAGE)/(double)TT); 
#endif
	  exit(1);
	}else{
	  register int n;

	  for(n=0; n<256; n++)
	    map_lut[n] = (u_char) colors[memo[n]].pixel;
	}
      }
	
      if((lr=recvfrom(sock, (char *)buf, T_MAX, 0, &addr_from, &fromlen)) < 0){
	perror("VideoDecode'ReadBuffer'recvfrom");
	continue;
      }
      if((type=GetType(buf)) != VIDEO_TYPE){
	fprintf(stderr, "Received and dropped a packet type %d\n", type);
	continue;
      }
      if(addr_from.sin_addr.s_addr != host_sin_addr){
#ifdef DEBUG
	fprintf(stderr, "VideoDecode: Received and dropped a video packet\n");
#endif
	continue;
      }

      /* Header video packet format : 3 bytes
      *  -----------------  00: Video Packet header data.
      *  |0|0|I|P|S|N|N|C|  I : 1 if Full Intra Encoded image && first packet.
      *  -----------------  P : 1 if PAL FORMAT.
      *  | packet number |  S : 1 if CIF4, else 0.
      *  -----------------  NN: if CIF4, refers the CIF in the CIF4 (0 to 3). 
      *  | image  number |  C : 1 if color encoded, else 0.
      *  -----------------
      */

      packet_number = buf[1] & 0xff;

      if(first){
	COLOR_ENCODED = buf[0] & 0x01;
	SUPER_CIF = ((buf[0] & 0x08) ? TRUE : FALSE);
	PAL_FORMAT = ((buf[0] & 0x10) ? TRUE : FALSE);
	first = FALSE;
      }
      FULL_INTRA = ((buf[0] & 0x20) ? TRUE : FALSE);
      if(FULL_INTRA_REQUEST){
	if(!FULL_INTRA){
	  if(nb_requests % 10 == 0)
	    SendFullIntraRequest(host_coder);
	  nb_requests ++;
	  continue;
	}else{
	  FULL_INTRA_REQUEST = FALSE;
	  packet_expected = (packet_number + 1) % 256;
	  break;
	}
      }else{
	if(packet_expected == packet_number){
	  break;
	}else{
	  register int i;
	  int temp;

	  for(i=0; i<10; i++){
	    temp = (packet_number + i) % 256;
	    if(temp == packet_expected)
	      break;
	  }
	  if(i != 10)
	    /* This packet is an old packet, Threw it away ... */
	    continue;

	  for(i=0; i<10; i++){
	    temp = (packet_expected + i) % 256;
	    if(temp == packet_number)
	      /* Some packets have been lost, continue decoding with
	      *  this new packet.
	      */
	      break;
	  }
	  if(i == 10)
	    /* There are a lot of lost packets, Send a Full INTRA request */
	    SendFullIntraRequest(host_coder);
	  /* Continue decoding with this new packet */
	  break;
	}
      }
    }while(1);

    if(SUPER_CIF)
      cif_number = (buf[0] >> 1) & 0x03;
    image_number = buf[2] & 0xff;
    packet_expected = (packet_number + 1) % 256;
    *i = 3;
    return(lr);
  }


    

VideoDecode(fd, port, group, name, sin_addr, display, QCIF2CIF)
     int fd[2];
     char *port, *group;
     char *name; /* Window's name. */
     u_long sin_addr;
     char *display;
     BOOLEAN QCIF2CIF; /* True if CIF display. */
    

{
    u_char *im_pix;
    int RT=0; /* Temporal reference */
    BOOLEAN PART_ECRAN, CAMERA_DOC, GEL_IMAGE; 
    BOOLEAN CIF; /* True if CIF format */
    BOOLEAN QUANTM, DVM, CBC, COEFFT, FIL; /* booleans for TYPEM value */
    BOOLEAN P; /* Booleans for CBC : P&0x01->Cr, .., P&0x20->Y1 */
    BOOLEAN FIRST=TRUE; /* True if it is the first image. */
    BOOLEAN GO=TRUE;
    BOOLEAN DOUBLE=FALSE; /* True if QCIF && QCIF2CIF */
    BOOLEAN ERROR_BIT=FALSE; /* True if a bit error is detected */
    int NG; /* Current GOB number*/
    int quant; /* Current quantizer */
    int mba; /* Current macroblock number */
    int x, new_x, new_bit, result;
    BLOCK8x8 block; /* current DCT coefficients block */
    int *tab_quant1, *tab_quant2;
    int x_mb, y_mb; /* MB's offset */
    int x_mb2, y_mb2;
    int L_col, L_lig;
    int offL_lig;
    int NG_MAX; /* Maximum NG value */
    u_char chaine_codee[T_MAX];
    int i = 0; /* Current byte of chaine_codee. */
    int j = 0; /* Current bit of i */
    int memo_RT;
    register int l;
    struct sockaddr_in addr;
    int len_addr;
    char *pt;
    int NG_EXPECTED=0; /* Next GOB number expected */
    int full_cbc;
    int temp;

    fd_pipe = fd[0];
    sock=CreateReceiveSocket(&addr, &len_addr, port, group);
    pt = (char *)index(name, '@');
    strcpy(host_coder, (char *) (pt+1));
    host_sin_addr = sin_addr;

    do{
      lmax=ReadBuffer(sock, chaine_codee, &i);
      temp=get_bit(chaine_codee,i,j);
      for(l=1; l<20; l++)
	temp= (temp<<1) + get_bit(chaine_codee,i,j);
      if(temp != 16){
	fprintf(stderr,"decode: Bad First Header Image value\n");
	SendFullIntraRequest(host_coder);
      }
    }while(temp != 16);
    NG=0;
/*
	  gettimeofday(&realtime, (struct timezone *)NULL);
	  inittime = realtime.tv_sec + realtime.tv_usec/1000000.0;
*/
    /* Using two separate modules is better for CIF & QCIF 
    *  coding performances. Avoid many reference pointers...
    */

    if(SUPER_CIF){
      /*--------------------------------------------------------------------*\
      *                        SUPER CIF ENCODING                            *
      \*--------------------------------------------------------------------*/
      static u_char tab_Y[4][288][352], tab_Cb[4][144][176], 
                    tab_Cr[4][144][176];
      static int L_lig2, L_col2;

      do{ /* IMAGE LOOP */

	do{ /* GOB LOOP */

	  if(NG == 0){
	    /*************\
	    * IMAGE LAYER *
	    \*************/

	    memo_RT = RT;
	    RT = get_bit(chaine_codee,i,j);
	    for(l=1; l<5; l++)
	      RT = (RT<<1) + get_bit(chaine_codee,i,j);
	
	    PART_ECRAN = get_bit(chaine_codee,i,j);
	    CAMERA_DOC = get_bit(chaine_codee,i,j);
	    GEL_IMAGE = get_bit(chaine_codee,i,j);
	    CIF = get_bit(chaine_codee,i,j);
	    if(FIRST){
	      L_col = 352;
	      if(PAL_FORMAT)
		L_lig = 288;
	      else
		L_lig = 240;
	      L_col2 = L_col << 1;
	      L_lig2 = L_lig << 1;
	      offL_lig=L_lig-3;
	      if(PAL_FORMAT){
		NG_MAX = 12;
	      }else{
		NG_MAX = 10;
	      }
	      strcat(name, " Super CIF"); 
	      init_window(L_lig2, L_col2, name, map_lut, display,
			  COLOR_ENCODED,  &depth, &dpy, &ximage, colors);
	      im_pix = (u_char *) ximage->data;
	      full_cbc = (COLOR_ENCODED ? 63 : 60);
	      FIRST = FALSE;
	    }
	    temp = get_bit(chaine_codee,i,j); /* reserved */
	    temp = get_bit(chaine_codee,i,j); /* reserved */
	    while (get_bit(chaine_codee,i,j))
	      /* RESERVEI option unused */
	      for(l=0; l<8; l++)
		temp = get_bit(chaine_codee,i,j);
	    temp = get_bit(chaine_codee,i,j);
	    for(l=1; l<16; l++)
	      temp = (temp<<1) + get_bit(chaine_codee,i,j);
	    if(temp != 1){
	      fprintf(stderr,"decode: Bad Header GOB value after image\n");
	      exit(-1);
	    }
	    if(NG != NG_EXPECTED){
	      int image, cif;

	      cif = (cif_number == 0 ? 3 : cif_number-1);
	      image = (cif == 0 ? (image_number == 0 ? 255 : image_number-1) : 
		       image_number);
#ifdef DEBUG
	      fprintf(stderr, 
            "Send Packet loss Info: im:#%d, cif:#%d, gobfrom:#%d, gobto:#%d\n",
		      image, cif, NG_EXPECTED, NG_MAX);
#endif
	    SendInfoPacketLost(host_coder, NG_EXPECTED, NG_MAX, image, cif);
	    }
	    NG_EXPECTED = 1;
	    NG = get_bit(chaine_codee,i,j);
	    for(l=1; l<4; l++)
	      NG = (NG<<1) + get_bit(chaine_codee,i,j);
	    continue;
	  }
	  /***********************\
	  * GROUP OF BLOCKS LAYER *
	  \***********************/

	  if(NG != NG_EXPECTED){
	    if(NG > NG_MAX){
	      fprintf(stderr,"decode: Bad Header GOB value, NG=%d\n", NG);
	      exit(-1);
	    }
	    if(NG_EXPECTED == 0)
	      NG_EXPECTED = 1;
	  
	    if(NG < NG_EXPECTED){
	      int image, cif;

	      cif = (cif_number == 0 ? 3 : cif_number-1);
	      image = (cif == 0 ? (image_number == 0 ? 255 : image_number-1) : 
		       image_number);
#ifdef DEBUG
	      fprintf(stderr, 
		      "Send Packet loss Info: im:#%d, cif:#%d, gobfrom:#%d, gobto:#%d AND: \n",
		      image, cif, NG_EXPECTED, NG_MAX);
#endif
	      SendInfoPacketLost(host_coder, NG_EXPECTED, NG_MAX, image, cif);
	      NG_EXPECTED = 1;
	    }

#ifdef DEBUG
	    fprintf(stderr, 
            "Send Packet loss Info: im:#%d, cif:#%d, gobfrom:#%d, gobto:#%d\n",
		    image_number, cif_number, NG_EXPECTED, NG-1);
#endif
	    SendInfoPacketLost(host_coder, NG_EXPECTED, NG-1, image_number, 
			       cif_number);
	  }
	  quant = get_bit(chaine_codee,i,j);
	  for(l=1; l<5; l++)
	    quant = (quant<<1) + get_bit(chaine_codee,i,j);
	  tab_quant1 = tab_rec1[quant-1];
	  tab_quant2 = tab_rec2[quant-1];
	  while (get_bit(chaine_codee,i,j))
	    /* RESERVEG option unused */
	    for(l=0; l<8; l++)
	      temp = get_bit(chaine_codee,i,j); 

	  /******************\
	  * MACROBLOCK LAYER *
	  \******************/

	  mba = 0;
	  do{ /* MB LOOP */

	    /* Looking for MBA value */
	    do{
	      new_x = 0;
	      do{
		x = new_x;
		new_bit = get_bit(chaine_codee,i,j);
		new_x = state_mba[x].bit[new_bit];
	      }while(new_x != 0);
	      if((result = state_mba[x].result[new_bit]) > 0){
		mba += result;
		break;
	      }else{
		if(result == PADDING)
		  continue;
		if(result == CDI)
		  break;
		else{
		  ERROR_BIT=TRUE;
		  break;
		}
	      }
	    }while(1);

	    if(ERROR_BIT || (mba > 33)){
#ifdef DEBUG
	      fprintf(stderr, "Bit error detected\n");
#endif
	      RetrieveBitError(chaine_codee, &i, &j);
	      ERROR_BIT = FALSE;
	      break; /* Out of the MB loop with the new GOB */
	    }
	    if(result == CDI)
	      break;
	    
	    x_mb = MBcol[NG][mba];
	    y_mb = MBlig[NG][mba];

	    x_mb2 = x_mb >> 1;
	    y_mb2 = y_mb >> 1;

	    /* Looking for TYPEM value */
	    new_x = 0;
	    do{
	      x = new_x;
	      new_bit = get_bit(chaine_codee,i,j);
	      new_x = state_typem[x].bit[new_bit];
	    }while(new_x != 0);
	    if((result = state_typem[x].result[new_bit]) > 0){
	      QUANTM = result & 0x10;
	      DVM = result & 0x08;
	      CBC = result & 0x04;
	      COEFFT = result & 0x02;
	      FIL = result & 0x01;
	    }else{
	      fprintf(stderr, "Bad Value of CLV_TYPEM\n");
	      exit(-2);
	    }
	
	    if(QUANTM){
	      quant = get_bit(chaine_codee,i,j);
	      for(l=1; l<5; l++)
	        quant = (quant<<1) + get_bit(chaine_codee,i,j);
	      tab_quant1 = tab_rec1[quant-1];
	      tab_quant2 = tab_rec2[quant-1];
	    }
	    if(CBC){
	      /* Looking for CBC value */
	      new_x = 0;
	      do{
		x = new_x;
		new_bit = get_bit(chaine_codee,i,j);
		new_x = state_cbc[x].bit[new_bit];
	      }while(new_x != 0);
	      P = state_cbc[x].result[new_bit];
	    }else
	      P = full_cbc;

	    /*************\
	    * BLOCK LAYER *
	    \*************/
	    if(COEFFT){
	      int cm; /* block[][cm+1] = 0 */
	      /* Block decoding in the order :
	      *  Y1, Y2, Y3, and Y4. If COLOR_ENCODED, Cb and Cr are encoded.
	      */
	      if(P & 0x20){
		decode_bloc_8x8(chaine_codee, block, state_tcoeff, &i, &j, 
				&cm, CBC);
		ifct_8x8(block, tab_quant1, tab_quant2, CBC, 
			 &(tab_Y[cif_number][y_mb][x_mb]), cm, L_Y);
	      }
	      if(P & 0x10){
		decode_bloc_8x8(chaine_codee, block, state_tcoeff, &i, &j, 
				&cm, CBC);
		ifct_8x8(block, tab_quant1, tab_quant2, CBC, 
			 &(tab_Y[cif_number][y_mb][x_mb+8]), cm, L_Y);
	      }
	      if(P & 0x08){
		decode_bloc_8x8(chaine_codee, block, state_tcoeff, &i, &j, 
				&cm, CBC);
		ifct_8x8(block, tab_quant1, tab_quant2, CBC, 
			 &(tab_Y[cif_number][y_mb+8][x_mb]), cm, L_Y);
	      }
	      if(P & 0x04){
		decode_bloc_8x8(chaine_codee, block, state_tcoeff, &i, &j, 
				&cm, CBC);
		ifct_8x8(block, tab_quant1, tab_quant2, CBC, 
			 &(tab_Y[cif_number][y_mb+8][x_mb+8]), cm, L_Y);
	      }
	      if(COLOR_ENCODED){
		if(P & 0x02){
		  decode_bloc_8x8(chaine_codee, block, state_tcoeff, &i, &j, 
				  &cm, CBC);
/*		  ifct_8x8(block, tab_quant1, tab_quant2, CBC, 
			   &(tab_Cb[cif_number][y_mb2][x_mb2]), cm, L_Y_DIV2);*/
		}
		if(P & 0x01){
		  decode_bloc_8x8(chaine_codee, block, state_tcoeff, &i, &j, 
				  &cm, CBC);
/*		  ifct_8x8(block, tab_quant1, tab_quant2, CBC, 
			   &(tab_Cr[cif_number][y_mb2][x_mb2]), cm, L_Y_DIV2);*/
		}
	      }
	    }

	    if(COLOR_ENCODED){

	      /* COLOR MACROBLOCK ENCODING ...
	      */
	      int l, c;
	      register int x, y, yy, xd2, yd2;
/*	  
	      if(depth == 8){
		l = y_mb+16;
		c = x_mb+16;
		for(y=y_mb, yy=y*L_col; y<l; y++, yy+=L_col){
		  yd2 = y >> 1;
		  for(x=x_mb; x<c; x++){
		    xd2 = x >> 1;
		    im_pix[yy+x] = map_lut[YUV_to_RVB[(tab_Y[cif_number][y][x]>>3)][(tab_Cb[cif_number][yd2][xd2]>>4)][(tab_Cr[cif_number][yd2][xd2]>>4)]];
		  }
		}
	      }else{
	      }
*/
	    }else{

	      /* B&W MACROBLOCK ENCODING (Y -> Pixel)
	      *  One pixel is encoded with 6 bits (64 shades of gray)
	      */
	      register int l, c, offset1, offset2;
	      register u_char *tab_Y_x, *im_pix_x;

	      if(depth == 4){
		offset1 = L_Y - 16;
		offset2 = (L_col2 - 16)>>1;
		tab_Y_x = &(tab_Y[cif_number][y_mb][x_mb]);
		switch(cif_number){
		case 0 : 
		  im_pix_x = &(im_pix[(x_mb + (y_mb*L_col2))>>1]); 
		  break;
		case 1 : 
		  im_pix_x = &(im_pix[(x_mb + (1+(y_mb<<1))*L_col)>>1]); 
		  break;
		case 2 : 
		  im_pix_x = &(im_pix[(x_mb + (offL_lig+y_mb)*L_col2)>>1]); 
		  break;
		case 3 : 
		  im_pix_x = &(im_pix[(x_mb + (1+((offL_lig+y_mb)<<1))*L_col)
				      >>1]);
		  break;
		}
		if((cif_number<2) && (NG > 10) && (mba>22)){
		  for(l=0; l<13; l++){
		    for(c=0; c<8; c++){
		      *im_pix_x++ = map_lut[((*tab_Y_x++ >> 4) << 4) |
					    (*tab_Y_x++ >> 4)];
		    }
		    im_pix_x += offset2;
		    tab_Y_x  += offset1;
		  }
		}else{
		  for(l=0; l<16; l++){
		    for(c=0; c<8; c++){
		      *im_pix_x++ = map_lut[((*tab_Y_x++ >> 4) << 4) |
					    (*tab_Y_x++ >> 4)];
		    }
		    im_pix_x += offset2;
		    tab_Y_x  += offset1;
		  }
		}
	      }else{
		offset1 = L_Y - 16;
		offset2 = L_col2 - 16;
		tab_Y_x = &(tab_Y[cif_number][y_mb][x_mb]);
		switch(cif_number){
		case 0 : 
		  im_pix_x = &(im_pix[x_mb + (y_mb*L_col2)]); 
		  break;
		case 1 : 
		  im_pix_x = &(im_pix[x_mb + (1+(y_mb<<1))*L_col]); 
		  break;
		case 2 :
		    im_pix_x = &(im_pix[x_mb + (offL_lig+y_mb)*L_col2]);
		  break;
		case 3 : 
		  im_pix_x = &(im_pix[x_mb + (1+((offL_lig+y_mb)<<1))*L_col]);
		  break;
		}
		if((cif_number<2) && (NG > 10) && (mba>22)){
		  for(l=0; l<13; l++){
		    for(c=0; c<16; c++)
		      *im_pix_x++ = map_lut[*tab_Y_x++];
		    im_pix_x += offset2;
		    tab_Y_x  += offset1;
		  }
		}else{
		  for(l=0; l<16; l++){
		    for(c=0; c<16; c++)
		      *im_pix_x++ = map_lut[*tab_Y_x++];
		    im_pix_x += offset2;
		    tab_Y_x  += offset1;
		  }
		}
	      }
	    }
	  }while(1); /* MB LOOP */

	  NG_EXPECTED = (NG==NG_MAX ? 0 : NG+1);
	  NG=get_bit(chaine_codee,i,j);
	  for(l=1; l<4; l++)
	  NG = (NG<<1) + get_bit(chaine_codee,i,j);
	  if(NG == 0){
	    /* It is the next image's header */
	    GO = FALSE;
	    /* Display the image */
	    if(cif_number == 0){
	      NB_IMAGE ++;
	      show(dpy, ximage, L_col2, L_lig2);
	    }
	  }
	  
	}while(GO); /* GOB LOOP */
	
	GO = TRUE;
	
      }while(1); /* IMAGE LOOP */

    }else{
      /*------------------------------------------------------------------*\
      *                      CIF or QCIF encoding                          *
      \*------------------------------------------------------------------*/

      int INC_NG;
      static u_char tab_Y[288][352], tab_Cb[144][176], tab_Cr[144][176];

      do{ /* IMAGE LOOP */

	do{ /* GOB LOOP */

	  if(NG == 0){
	    /*************\
	    * IMAGE LAYER *
	    \*************/

	    BOOLEAN NEW_FORMAT=FALSE;
	    BOOLEAN LAST_CIF=CIF;
      
	    memo_RT = RT;
	    RT = get_bit(chaine_codee,i,j);
	    for(l=1; l<5; l++)
	      RT = (RT<<1) + get_bit(chaine_codee,i,j);
	
	    PART_ECRAN = get_bit(chaine_codee,i,j);
	    CAMERA_DOC = get_bit(chaine_codee,i,j);
	    GEL_IMAGE = get_bit(chaine_codee,i,j);
	    CIF = get_bit(chaine_codee,i,j);
	    if(!FIRST && (LAST_CIF != CIF))
	      NEW_FORMAT = TRUE;
	    if(FIRST || NEW_FORMAT){
	      if(CIF){
		L_col = 352;
		L_lig = (PAL_FORMAT ? 288 : 240);
		NG_MAX = (PAL_FORMAT ? 12 : 10);
		strcat(name, " CIF");
		INC_NG = 1;
	      }
	      else{
		if(QCIF2CIF){
		  DOUBLE=1;
		  strcat(name, " QCIF -> CIF");
		  L_col = 352;
		  L_lig = (PAL_FORMAT ? 288 : 224);
		}else{
		  strcat(name, " QCIF");
		  L_col = 176;
		  L_lig = (PAL_FORMAT ? 144 : 112);
		}
		NG_MAX = 5;
		INC_NG = 2;
	      }
	      if(NEW_FORMAT){
		/* New format detected */
		fprintf(stderr, "new format detected\n");
		kill_window(dpy, ximage);
		if(!FULL_INTRA){
		  /* Waiting for Full INTRA image */
		  i = j = 0;
		  FULL_INTRA_REQUEST = TRUE;
		  lmax = ReadBuffer(sock, chaine_codee, &i);
		}
	      }
	      init_window(L_lig, L_col, name, map_lut, display, COLOR_ENCODED,
			  &depth, &dpy, &ximage, colors);
	      im_pix = (u_char *) ximage->data;
	      full_cbc = (COLOR_ENCODED ? 63 : 60);
	      FIRST = FALSE;
	    }
	    temp = get_bit(chaine_codee,i,j); /* reserved */
	    temp = get_bit(chaine_codee,i,j); /* reserved */
	    while (get_bit(chaine_codee,i,j))
	      /* RESERVEI option unused */
	      for(l=0; l<8; l++)
		temp = get_bit(chaine_codee,i,j);
	    temp = get_bit(chaine_codee,i,j);
	    for(l=1; l<16; l++)
	      temp = (temp<<1) + get_bit(chaine_codee,i,j);
	    if(temp != 1){
	      fprintf(stderr, "decode: Bad Header GOB value after image\n");
	      exit(-1);
	    }
	    if(NG != NG_EXPECTED){
	      int image;

	      image = (image_number == 0 ? 255 : image_number-1);
#ifdef DEBUG
	      fprintf(stderr, 
            "Send Packet loss Info: im:#%d, cif:#%d, gobfrom:#%d, gobto:#%d\n",
		      image, cif_number, NG_EXPECTED, NG_MAX);
#endif
	    SendInfoPacketLost(host_coder, NG_EXPECTED, NG_MAX, image, 
			       cif_number);
	    }
	    NG_EXPECTED = 1;
	    NG = get_bit(chaine_codee,i,j);
	    for(l=1; l<4; l++)
	      NG = (NG<<1) + get_bit(chaine_codee,i,j);
	    continue;
	  }

	  /***********************\
	  * GROUP OF BLOCKS LAYER *
	  \***********************/

	  if(NG != NG_EXPECTED){
	    if(NG > NG_MAX){
	      fprintf(stderr, "decode: Bad Header GOB value, NG=%d\n", NG);
	      exit(-1);
	    }
	    if(NG_EXPECTED == 0)
	      NG_EXPECTED = 1;

	    if(NG < NG_EXPECTED){
	      int image;

	      image = (image_number == 0 ? 255 : image_number-1);
#ifdef DEBUG
	      fprintf(stderr, 
		"Send Packet loss Info: im:#%d, gobfrom:#%d, gobto:#%d AND: \n",
		      image, NG_EXPECTED, NG_MAX);
#endif
	      SendInfoPacketLost(host_coder, NG_EXPECTED, NG_MAX, image, 
				 cif_number);
	      NG_EXPECTED = 1;
	    }

#ifdef DEBUG
	    fprintf(stderr, 
            "Send Packet loss Info: im:#%d, cif:#%d, gobfrom:#%d, gobto:#%d\n",
		    image_number, cif_number, NG_EXPECTED, NG-INC_NG);
#endif
	    SendInfoPacketLost(host_coder, NG_EXPECTED, NG-INC_NG, 
			       image_number, cif_number);
	  }
	  quant = get_bit(chaine_codee,i,j);
	  for(l=1; l<5; l++)
	    quant = (quant<<1) + get_bit(chaine_codee,i,j);
	  tab_quant1 = tab_rec1[quant-1];
	  tab_quant2 = tab_rec2[quant-1];
	  while (get_bit(chaine_codee,i,j))
	    /* RESERVEG option unused */
	    for(l=0; l<8; l++)
	      temp = get_bit(chaine_codee,i,j); 

 	  /******************\
          * MACROBLOCK LAYER *
	  \******************/

	  mba = 0;
	  do{ /* MB LOOP */

	    /* Looking for MBA value */
	    do{
	      new_x = 0;
	      do{
		x = new_x;
		new_bit = get_bit(chaine_codee,i,j);
		new_x = state_mba[x].bit[new_bit];
	      }while(new_x != 0);
	      if((result = state_mba[x].result[new_bit]) > 0){
		mba += result;
		break;
	      }else{
		if(result == PADDING)
		  continue;
		if(result == CDI)
		  break;
		else{
		  ERROR_BIT=TRUE;
		  break;
		}
	      }
	    }while(1);

	    if(ERROR_BIT || (mba > 33)){
#ifdef DEBUG
	      fprintf(stderr, "Bit error detected\n");
#endif
	      RetrieveBitError(chaine_codee, &i, &j);
	      ERROR_BIT = FALSE;
	      break; /* Out of the MB loop with the new GOB */
	    }
	    if(result == CDI)
	      break;
	    
	    x_mb = MBcol[NG][mba];
	    y_mb = MBlig[NG][mba];

	    x_mb2 = x_mb >> 1;
	    y_mb2 = y_mb >> 1;

	    /* Looking for TYPEM value */
	    new_x = 0;
	    do{
	      x = new_x;
	      new_bit = get_bit(chaine_codee,i,j);
	      new_x = state_typem[x].bit[new_bit];
	    }while(new_x != 0);
	    if((result = state_typem[x].result[new_bit]) > 0){
	      QUANTM = result & 0x10;
	      DVM = result & 0x08;
	      CBC = result & 0x04;
	      COEFFT = result & 0x02;
	      FIL = result & 0x01;
	    }else{
	      fprintf(stderr,"Bad Value of CLV_TYPEM\n");
	      exit(-2);
	    }
	
	    if(QUANTM){
	      quant = get_bit(chaine_codee,i,j);
	      for(l=1; l<5; l++)
	        quant = (quant<<1) + get_bit(chaine_codee,i,j);
	      tab_quant1 = tab_rec1[quant-1];
	      tab_quant2 = tab_rec2[quant-1];
	    }
	    if(CBC){
	      /* Looking for CBC value */
	      new_x = 0;
	      do{
		x = new_x;
		new_bit = get_bit(chaine_codee,i,j);
		new_x = state_cbc[x].bit[new_bit];
	      }while(new_x != 0);
	      P = state_cbc[x].result[new_bit];
	    }else
	      P = full_cbc;

	    /*************\
	    * BLOCK LAYER *
	    \*************/
	    if(COEFFT){
	      int cm; /* block[][cm+1] = 0 */
	      /* Block decoding in the order :
	      *  Y1, Y2, Y3, and Y4. If COLOR_ENCODED, Cb and Cr are encoded.
	      */
	      if(P & 0x20){
		decode_bloc_8x8(chaine_codee, block, state_tcoeff, &i, &j, 
				&cm, CBC);
		ifct_8x8(block, tab_quant1, tab_quant2, CBC, 
			 &(tab_Y[y_mb][x_mb]), cm, L_Y);
	      }
	      if(P & 0x10){
		decode_bloc_8x8(chaine_codee, block, state_tcoeff, &i, &j, 
				&cm, CBC);
		ifct_8x8(block, tab_quant1, tab_quant2, CBC, 
			 &(tab_Y[y_mb][x_mb+8]), cm, L_Y);
	      }
	      if(P & 0x08){
		decode_bloc_8x8(chaine_codee, block, state_tcoeff, &i, &j, 
				&cm, CBC);
		ifct_8x8(block, tab_quant1, tab_quant2, CBC, 
			 &(tab_Y[y_mb+8][x_mb]), cm, L_Y);
	      }
	      if(P & 0x04){
		decode_bloc_8x8(chaine_codee, block, state_tcoeff, &i, &j, 
				&cm, CBC);
		ifct_8x8(block, tab_quant1, tab_quant2, CBC, 
			 &(tab_Y[y_mb+8][x_mb+8]), cm, L_Y);
	      }
	      if(COLOR_ENCODED){
		if(P & 0x02){
		  decode_bloc_8x8(chaine_codee, block, state_tcoeff, &i, &j, 
				  &cm, CBC);
		  ifct_8x8(block, tab_quant1, tab_quant2, CBC, 
			   &(tab_Cb[y_mb2][x_mb2]), cm, L_Y_DIV2);
		}
		if(P & 0x01){
		  decode_bloc_8x8(chaine_codee, block, state_tcoeff, &i, &j, 
				  &cm, CBC);
		  ifct_8x8(block, tab_quant1, tab_quant2, CBC, 
			   &(tab_Cr[y_mb2][x_mb2]), cm, L_Y_DIV2);
		}
	      }
	    }
	    
	    if(COLOR_ENCODED){
	      /* COLOR MACROBLOCK ENCODING ...
	      */
	      int l, c;
	      register int x, y, yy, xd2, yd2;
	  
	      if(depth == 8){
		l = y_mb+16;
		c = x_mb+16;
		for(y=y_mb, yy=y*L_col; y<l; y++, yy+=L_col){
		  yd2 = y >> 1;
		  for(x=x_mb; x<c; x++){
		    xd2 = x >> 1;
		    im_pix[yy+x] = map_lut[yuv_to_lut[((tab_Y[y][x]&0xf0)<<2)|
                   ((tab_Cb[yd2][xd2]&0xe0)>>2)|((tab_Cr[yd2][xd2]&0xe0)>>5)]];
		  }
		}
	      }else{
	      }

	    }else{

	      /* B&W MACROBLOCK ENCODING (Y -> Pixel)
	      *  One pixel is encoded with 6 bits (64 shades of gray)
	      */
	      if(DOUBLE){
		register int offset1, offset2;
		register int l, c;
		register u_char aux, *tab_Y_x, *im_pix1, *im_pix2, temp;

		if(depth == 4){
		  offset1 = L_Y - 16;
		  offset2 = L_col - 16;
		  tab_Y_x = &(tab_Y[y_mb][x_mb]);
		  im_pix1 = &(im_pix[(y_mb*L_col)+x_mb]);
		  im_pix2 = &(im_pix[((y_mb+1)*L_col)+x_mb]);
		  for(l=0; l<16; l++){
		    for(c=0; c<16; c++){
		      temp = (*tab_Y_x>>4);
		      aux=map_lut[(temp<<4) + temp];
		      tab_Y_x ++;
		      *im_pix1++ = aux;
		      *im_pix2++ = aux;
		    }
		    tab_Y_x += offset1;
		    im_pix1 += offset2;
		    im_pix2 += offset2;
		  }
		}else{
		  offset1 = L_Y - 16;
		  offset2 = (L_col<<1) - 32;
		  tab_Y_x = &(tab_Y[y_mb][x_mb]);
		  im_pix1 = &(im_pix[(y_mb*L_col + x_mb)<<1]);
		  im_pix2 = &(im_pix[(((y_mb<<1)+1)*L_col)+(x_mb<<1)]);
		  for(l=0; l<16; l++){
		    for(c=0; c<16; c++){
		      aux=map_lut[*tab_Y_x++];
		      *im_pix1++ = aux;
		      *im_pix1++ = aux;
		      *im_pix2++ = aux;
		      *im_pix2++ = aux;
		    }
		    tab_Y_x += offset1;
		    im_pix1 += offset2;
		    im_pix2 += offset2;
		  }
		}
	      }else{
		register int l, c, offset1, offset2;
		register u_char *tab_Y_x, *im_pix_x;

		if(depth == 4){
		  offset1 = L_Y - 16;
		  offset2 = (L_col - 16)>>1;
		  tab_Y_x = &(tab_Y[y_mb][x_mb]);
		  im_pix_x = &(im_pix[((y_mb*L_col)+x_mb)>>1]);
		  for(l=0; l<16; l++){
		    for(c=0; c<8; c++){
		      *im_pix_x++ = map_lut[((*tab_Y_x++ >> 4) << 4) | 
					    (*tab_Y_x++ >> 4)];
		    }
		    im_pix_x += offset2;
		    tab_Y_x  += offset1;
		  }
		}else{
		  offset1 = L_Y - 16;
		  offset2 = L_col - 16;
		  tab_Y_x = &(tab_Y[y_mb][x_mb]);
		  im_pix_x = &(im_pix[(y_mb*L_col)+x_mb]);
		  for(l=0; l<16; l++){
		    for(c=0; c<16; c++)
		      *im_pix_x++ = map_lut[*tab_Y_x++];
		    im_pix_x += offset2;
		    tab_Y_x  += offset1;
		  }
		}
	      }
	    }

	  }while(1); /* MB LOOP */

	  NG_EXPECTED = (NG==NG_MAX ? 0 : NG+INC_NG);
	  NG=get_bit(chaine_codee,i,j);
	  for(l=1; l<4; l++)
	  NG = (NG<<1) + get_bit(chaine_codee,i,j);
	  if(NG == 0){
	    /* It's the next image's header */
	    GO = FALSE;
	    /* Display the image */
	    NB_IMAGE++;
	    show(dpy, ximage, L_col, L_lig);
	  }
	  
	}while(GO); /* GOB LOOP */
	
	GO = TRUE;
	
      }while(1); /* IMAGE LOOP */
    }
  } /* VideoDecode() */
